/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "auxiliarymethods.h"

AuxiliaryMethods::AuxiliaryMethods(QObject *parent) : QObject(parent){
}

AuxiliaryMethods::~AuxiliaryMethods(){
}

void AuxiliaryMethods::errorMessage(QWidget *parentWidget, QString *error){
    QMessageBox *msg = new QMessageBox(parentWidget);
    msg->setIcon(QMessageBox::Critical);
    msg->setText(*error);
    msg->setStandardButtons(QMessageBox::Ok);
    msg->setDefaultButton(QMessageBox::Ok);
    msg->exec();
    delete msg;
    delete error;
}

bool AuxiliaryMethods::isInt(QString *text){
    if (text->length() <= 0)
        return false;
    QRegExp *regExp = new QRegExp("^\\d+([eE]-?\\d{1,3})?$");
    if (regExp->exactMatch(*text)){
        delete regExp;
        return true;
    }
    else{
        delete regExp;
        return false;
    }
}

bool AuxiliaryMethods::isDouble(QString *text){
    if (text->length() <= 0)
        return false;
    text->replace(",",".");
    QRegExp *regExp = new QRegExp("^-?\\d+([.,]\\d+)?([eE]-?\\d{1,3})?$");
    if (regExp->exactMatch(*text)){
        delete regExp;
        return true;
    }
    else{
        delete regExp;
        return false;
    }
}

double AuxiliaryMethods::round(const double &number, short decimalPlaces /*number of decimal places*/){
    double decimalPower[] = { 1.E0, 1.E1, 1.E2, 1.E3, 1.E4, 1.E5, 1.E6}; //	decimal powers

    if (number >= 0){
        /*double temp = number * decimalPlaces[decimalPower + 1];
        if ( ( (int)(floor(temp)) ) % 10 == 5 && ( temp - ( (int)(floor(temp)) ) ) == 0. )
            if ( ( ( (int)(floor(temp/10.)) ) % 10 ) % 2 == 0 )
                return floor(number * decimalPlaces[decimalPower]) / decimalPlaces[decimalPower];
            return floor(number * decimalPlaces[decimalPower] + 1) / decimalPlaces[decimalPower];*/
        return (floor(number * decimalPlaces[decimalPower] + 0.5)) / decimalPlaces[decimalPower];
    }
    /*double temp = number * decimalPlaces[decimalPower + 1] * -1.;
    if ( ( (int)(floor(temp)) ) % 10 == 5 && ( temp - ( (int)(floor(temp)) ) ) == 0. )
        if ( ( ( (int)(floor(temp/10.)) ) % 10 ) % 2 == 0 )
            return ceil(number * decimalPlaces[decimalPower]) / decimalPlaces[decimalPower];
        return ceil(number * decimalPlaces[decimalPower] - 1) / decimalPlaces[decimalPower];*/
    return (ceil(number * decimalPlaces[decimalPower] - 0.5)) / decimalPlaces[decimalPower];
}

wavelength* AuxiliaryMethods::validateAndParse_Wavelength(QString * text){
    if (text->length() <= 0)
            return 0;
    int posDecSep = text->lastIndexOf(","); //position of decimal separator
    if (posDecSep > 0)
            text->replace(posDecSep,1,".");
    QRegExp *regExp = new QRegExp("^(.+ - )?\\d+([.,]\\d+)?$");
    if (regExp->exactMatch(*text)){
        wavelength *toReturn = new wavelength;
        QStringList splitted = text->split(" - ", QString::SkipEmptyParts);
        if (splitted.size() > 1){
            toReturn->name = splitted.at(0);
            toReturn->value = splitted.at(1).toDouble();
        }
        else{
            toReturn->name = "";
            toReturn->value = splitted.at(0).toDouble();
        }
        delete regExp;
        delete text;
        if (toReturn->value > 0)
            return toReturn;
        else
            return 0;
    }
    else{
        delete regExp;
        delete text;
        return 0;
    }
}

//search/find wavelength-structure in "QVector<wavelength> *wavelengths" and return index of it, returns -1 if wavelength-structure cannot be found
//required for loading parameter-files and to avoid double entries/items in the ComboBox for chosing wavelengths
int AuxiliaryMethods::findWavelength(wavelength *toFind, QVector<wavelength> *wavelengths){
    for (int i = 0; i < wavelengths->size(); ++i)
        if ( (wavelengths->at(i).value == toFind->value) && (wavelengths->at(i).name == toFind->name))
            return i;
    return -1;
}
