/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef AUXILIARYMETHODS_H
#define AUXILIARYMETHODS_H

#include <QMessageBox>
#include <QRegExp>
#include <QString>
#include <QStringList>

#include <cmath>

#include "structs.h"

#include <QObject>

class AuxiliaryMethods : public QObject{

    Q_OBJECT

public:
    explicit AuxiliaryMethods(QObject *parent = 0);
    ~AuxiliaryMethods();

    static void errorMessage(QWidget *parentWidget, QString *error);

    static bool isInt(QString *text);
    static bool isDouble(QString *text);
    static double round(const double &number, short decimalPlaces /*number of decimal places*/);

    //read/parse wavelength-structure from QString and validate syntax; returns pointer to parsed wavelength or 0 (null pointer) if syntax is invalid;
    static wavelength* validateAndParse_Wavelength(QString * text);

    //search/find wavelength-structure in "QVector<wavelength> *wavelengths" and return index of it, returns -1 if wavelength-structure cannot be found
    //required for loading parameter-files and to avoid double entries/items in the ComboBox for chosing wavelengths
    static int findWavelength(wavelength *toFind, QVector<wavelength> *wavelengths);
};

#endif // AUXILIARYMETHODS_H
