/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "calculations.h"

//constructor including initialization of interpolation functions
Calculations::Calculations(QString *dataPath){

    QVector<QPointF> *temp = new QVector<QPointF>(0);
    int  skipLines = 4;
    QFile *file;
    QTextStream *in;

    //BEGIN IIcomC
    file = new QFile(*dataPath + "/IIcomC.txt");
    file->open(QIODevice::ReadOnly | QIODevice::Text);
    in = new QTextStream(file);
    for (int i = 0; i < skipLines; ++i)
        in->readLine();
    while (!(in->atEnd())){
        QString *line = new QString(in->readLine());
        QStringList splitted = line->split(";",QString::SkipEmptyParts);
        temp->append(QPointF(splitted.at(0).toDouble(),splitted.at(1).toDouble()));
        delete line;
    }
    file->close();
    delete in;
    delete file;
    sizeOf_IcomC = temp->size();
    IcomC_x = new float[sizeOf_IcomC];
    IcomC_y = new float[sizeOf_IcomC];
    for (int i = 0; i < temp->size(); ++i){
        IcomC_x[i] = temp->at(i).x();
        IcomC_y[i] = temp->at(i).y();
    }
    temp->clear();
    //END IIcomC

    //BEGIN fN
    file = new QFile(*dataPath + "/fN.txt");
    file->open(QIODevice::ReadOnly | QIODevice::Text);
    in = new QTextStream(file);
    for (int i = 0; i < skipLines; ++i)
        in->readLine();
    while (!(in->atEnd())){
        QString *line = new QString(in->readLine());
        QStringList splitted = line->split(";",QString::SkipEmptyParts);
        temp->append(QPointF(splitted.at(0).toDouble(),splitted.at(1).toDouble()));
        delete line;
    }
    file->close();
    delete in;
    delete file;
    sizeOf_fN = temp->size();
    fN_x = new float[sizeOf_fN];
    fN_y = new float[sizeOf_fN];
    for (int i = 0; i < temp->size(); ++i){
        fN_x[i] = temp->at(i).x();
        fN_y[i] = temp->at(i).y();
    }
    temp->clear();
    //END fN

    //BEGIN fO
    file = new QFile(*dataPath + "/fO.txt");
    file->open(QIODevice::ReadOnly | QIODevice::Text);
    in = new QTextStream(file);
    for (int i = 0; i < skipLines; ++i)
        in->readLine();
    while (!(in->atEnd())){
        QString *line = new QString(in->readLine());
        QStringList splitted = line->split(";",QString::SkipEmptyParts);
        temp->append(QPointF(splitted.at(0).toDouble(),splitted.at(1).toDouble()));
        delete line;
    }
    file->close();
    delete in;
    delete file;
    sizeOf_fO = temp->size();
    fO_x = new float[sizeOf_fO];
    fO_y = new float[sizeOf_fO];
    for (int i = 0; i < temp->size(); ++i){
        fO_x[i] = temp->at(i).x();
        fO_y[i] = temp->at(i).y();
    }
    temp->clear();
    //END fO

    delete temp;
    delete dataPath;

}

//destructor including release of memory
Calculations::~Calculations(){

    delete[] IcomC_x;
    delete[] IcomC_y;

    delete[] fN_x;
    delete[] fN_y;

    delete[] fO_x;
    delete[] fO_y;

}


//hypergeometrical function 2F1
complex<double> Calculations::hypgeo2F1(double a, double b, double c, complex<double> z){
    fcomplex a_ = Complex(a,0.);
    fcomplex b_ = Complex(b,0.);
    fcomplex c_ = Complex(c,0.);
    fcomplex z_ = Complex(z.real(), z.imag());
    fcomplex erg_ = hypgeo(a_,b_,c_,z_);
    complex<double> erg (erg_.r, erg_.i);
    return erg;
}


//interpolation of the form factor of carbon
double Calculations::fCpara(double s){
    return (0.289677 + 1.05075 * exp(-2.43905 * pow(s,2)) + 1.56165 * exp(-0.164166 * pow(s,2)) + 3.09995 * exp((-28.) * pow(s,2) / 4.));
}

//interpolation of the form factor of carbon with respect to anisotropy
double Calculations::fCperp(double dan, double s){
    return (0.289677 + 1.05075 * exp(-2.43905 * pow(s,2)) + 1.56165 * exp(-0.164166 * pow(s,2)) + 3.09995 * exp((-28. - dan) * pow(s,2) / 4.));
}

//interpolation of the form factor of nitrogen
double Calculations::fN(double s){
    float fN_y2[sizeOf_fN]; //sec. deriv. of interpol. funct.
    spline(fN_x, fN_y, sizeOf_fN, 1E32, 1E32, fN_y2);
    float *result = new float;
    splint(fN_x, fN_y, fN_y2, sizeOf_fN, s, result);
    double result_double = (*result);
    delete result;
    return result_double;
}

//interpolation of the form factor of oxygen
double Calculations::fO(double s){
    float fO_y2[sizeOf_fN]; //sec. deriv. of interpol. funct.
    spline(fO_x, fO_y, sizeOf_fO, 1E32, 1E32, fO_y2);
    float *result = new float;
    splint(fO_x, fO_y, fO_y2, sizeOf_fO, s, result);
    double result_double = (*result);
    delete result;
    return result_double;
}


//incoherent scattering

double Calculations::IcomC(double s){
    float IcomC_y2[sizeOf_fN]; //sec. deriv. of interpol. funct.
    spline(IcomC_x, IcomC_y, sizeOf_IcomC, 1E32, 1E32, IcomC_y2);
    float *result = new float;
    splint(IcomC_x, IcomC_y, IcomC_y2, sizeOf_IcomC, s, result);
    double result_double = (*result);
    delete result;
    return result_double;
}

double Calculations::Recoil(double s, double wavelength){
    double lam = wavelength;
    return (pow((lam / (lam + 0.0485 * pow(lam,2)/4. * pow(s,2))),3));
}

double Calculations::Qabs(double s, double wavelength){
    return (1./(1. + 0.0485 * wavelength * 3./8. * pow(s,2)));
}

double Calculations::Dq(double s){
    return (3.05 * pow(s,2) / (pow(0.53,2) + pow(s,2)));
}

double Calculations::Dlam(double s, double wavelength){
    return (pow(wavelength,2) / 137. * s * Dq(s));
}

double Calculations::Dlamc(double s, double wavelength){
    return ((0.0485 / 4. * pow(s,2) - 1.5E-3 * pow(s,4) / pow((pow(0.53,2) + pow(s,2)),2)) * pow(wavelength,2));
}

//attenuation function Q for compton scattering; due to secondary monochromation, pass-band b [angstr.]
double Calculations::Q(double b, double s, double wavelength){
    return (1 / (1 + Dlam(s,wavelength)/b) / (1. + pow(M_PI,2) * pow(Dlamc(s,wavelength),2) / pow((Dlam(s,wavelength) + b),2)));
}

double Calculations::IincC(bool useQ, double b, double s, double wavelength){
    return IcomC(s) * Recoil(s,wavelength) * Qabs(s,wavelength) * (useQ ? Q(b,s,wavelength) : 1);
}


//polarization factor P (P_ to avoid problems with one C macro)
double Calculations::P_(double s, double wavelength){
    return ((1 + pow(cos(2. * asin(wavelength * s / 2.)),2)) / 2.);
}


//I_intra - scattering from the layers

double Calculations::shk(int h, int k, double lcc){
    return (sqrt(pow(h,2) + pow(k,2) + h*k) * 2. / 3. / lcc);
}

double Calculations::g(double q, double lcc, int h, int k, double s){
    if (q == 0.)
        return 1;
    else
        if (s < shk(h, k, lcc))
            return (sqrt(q) / atanh(sqrt(q))/(1. + q));
        else
            return ((1. + q) * sqrt(q) / atanh(sqrt(q)) / ( pow((1. - q),2) + 4. * q * pow(shk(h, k, lcc),2)/pow(s,2) ));
}

double Calculations::g0(double q){
    if (q == 0)
        return 1;
    else
        return ((1. + q) * sqrt(q) / atanh(sqrt(q))/ pow((1. - q),2));
}

complex<double> Calculations::F(double m, double nu, double lm, double lcc, double sig1, int h, int k, double s){
    double al = nu / lm;
    double b = 2. * pow(M_PI, 2) * pow(shk(h, k, lcc),2) * pow(sig1,2) * 2./3. / lcc;
    complex<double> t (al+b, -2. * M_PI * s);
    double a_hypgeo = ((1+m)/2);
    double b_hypgeo = ((2+m)/2);
    double c_hypgeo = 1;
    complex<double> z_hypgeo ( (-1. * (4. * pow(M_PI,2) * pow(shk(h,k,lcc),2))),0);
    z_hypgeo /= pow(t,2);
    return ( pow(al,m) * pow(t,-1-m) * (double)(exp(gammln(1+m))) * hypgeo2F1(a_hypgeo, b_hypgeo, c_hypgeo, z_hypgeo));
}

//analytical expression for J_hk without epsilon_ 1
double Calculations::Jhk(double nu, double lm, double lcc, double sig1, int h, int k, double q, double s){
    int nu_round;
    if ( nu - floor(nu) == 0.5 )
        if (((int)(floor(nu))) % 2 == 0)
            nu_round = floor(nu);
        else
            nu_round = ceil(nu);
    else
        nu_round = floor(nu + 0.5);
    double t = g(q,lcc,h,k,s) * 1 / nu / s;
    complex<double> sum (0.,0.);
    for (int m = 0; m <= (nu_round  - 1); ++m){
        sum += ( ( (nu_round - m) / (double)(exp(factln(m))) ) * F(m,nu,lm,lcc,sig1,h,k,s) );
    }
    return t * sum.imag();
}

int Calculations::Jhk_prefactor(int h, int k){
    int multiplicity = 12;
    if ( (h == 0) && (k == 0))
        multiplicity = 1;
    else
        if ( (k == 0) || (h == k) )
            multiplicity = 6;
    int structFactSquared = 1;
    if ( ( (h-k) % 3) == 0 )
        structFactSquared = 4;
    return (multiplicity * structFactSquared);
}

double Calculations::JhkXprefactor(double nu, double lm, double lcc, double sig1, int h, int k, double q, double s){
    return (Jhk_prefactor (h, k) * Jhk(nu, lm, lcc, sig1, h, k, q, s)); // J_hk times prefactor
}

double Calculations::n0S0(double lcc){
    return ( pow(3.,(3./2.)) * pow (lcc,2));
}

double Calculations::Iintra(double nu, double lm, double lcc, double sig1, double q, double s, double wavelength){
    double Jhk_sum = 0.;
    if (wavelength <= 0.5)
        for (int h = 1; h <= 7; ++h)
            for (int k = 0; k <= h; ++k)
                Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, h, k, q, s);
    else{
        Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 1, 0, q, s);
        Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 1, 1, q, s);
        Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 2, 0, q, s);
        Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 2, 1, q, s);
        Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 3, 0, q, s);
        if (wavelength <= 1.25){
            Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 2, 2, q, s);
            Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 3, 1, q, s);
            Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 4, 0, q, s);
            if (wavelength <= 1.){
                Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 3, 2, q, s);
                Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 4, 1, q, s);
                Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 5, 0, q, s);
                if (wavelength <= 0.75){
                    Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 3, 3, q, s);
                    Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 4, 2, q, s);
                    Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 5, 1, q, s);
                    Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 6, 0, q, s);
                    Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 4, 3, q, s);
                    Jhk_sum += JhkXprefactor(nu, lm, lcc, sig1, 5, 2, q, s);
                }
            }
        }
    }
    return ( 1 / n0S0(lcc) * Jhk_sum );
}


//I_inter - scattering from the stacking of layers

complex<double> Calculations::H(double a3min, double da3, double sig3, double s){
    complex<double> i (0.,1.);
    return ( exp(2. * M_PI * i * a3min * s) / pow( ( 1. - 2. * M_PI * i * pow(sig3,2) * s / da3) ,( pow(da3,2) / pow(sig3,2) ) ) );
}

complex<double> Calculations::HN(double mu, double Nm, double a3min, double da3, double sig3, double s){
    return (pow( (1. - Nm * log(H(a3min,da3,sig3,s)) / mu ),-mu));
}

double Calculations::I1DI0(double mu, double Nm, double a3min, double da3, double sig3, double s){
    complex<double> HH;
    complex<double> HHN;
    HH = H(a3min,da3,sig3,s);
    HHN = HN(mu,Nm,a3min,da3,sig3,s);
    complex<double> t;
    t = ( (1. + HH)/(1. - HH) - 2. * HH * (1. - HHN) / Nm / pow((1. - HH),2) );
    return t.real();
}

double Calculations::I0(double mu, double Nm, double a3min, double da3, double s){
    double a3m = a3min + da3;
    return ( 1 /(2 * pow(M_PI,2) * pow(s,2) * Nm * pow(a3m,2)) *
             (1 - (cos(mu * atan(2 * M_PI * a3m * Nm * s / mu))) / (pow((1 + 4 * pow(M_PI,2) * pow (a3m,2) * pow(Nm,2) * pow(s,2) / pow(mu,2) ), (mu/2.))) ) );
}

double Calculations::I1D(double mu, double Nm, double a3min, double da3, double sig3, double s){
    return (I1DI0(mu,Nm,a3min,da3,sig3,s)-I0(mu, Nm, a3min, da3, s));
}

double Calculations::DD(double u3, double s){
    return (exp(-4. * pow(M_PI,2) * pow(u3,2) * pow(s,2)));
}

double Calculations::Iinter(double mu, double Nm, double a3min, double da3, double sig3, double u3, double eta, double lcc, double q, double s){
    return (g0(q) * 2 / M_PI / pow(s,2) / n0S0(lcc) * (1. + eta * DD(u3,s) * (I1D(mu,Nm,a3min,da3,sig3,s) -1)));
}


//I_coh - coherent scattering
double Calculations::Icoh(double cno, double mu, double Nm, double a3min, double da3, double sig3, double u3, double eta, double nu, double lm , double lcc, double sig1, double q, double cN, double cO, double dan, double s, double wavelength){
    return ( ( (1-cno) *
             ( (pow(fCperp(dan,s),2) * Iinter(mu, Nm, a3min, da3, sig3, u3, eta, lcc, q, s)) +
               (pow(fCpara(s),2) * Iintra(nu, lm, lcc, sig1, q, s, wavelength)) ) +
             (pow(fCpara(s),2) * cno)) +
             (pow(fN(s),2)*cN) + (pow(fO(s),2)*cO) );
}


//I_obs - theoretical observed intensity

//I_fit - fit function
double Calculations::Ifit(bool useQ, double b, double k, double cno, double mu, double Nm, double a3min, double da3, double sig3, double u3, double eta, double nu, double lm, double lcc, double sig1, double q, double cN, double cO, double dan, double s, double wavelength){
    return (k * P_(s,wavelength) * (Icoh(cno, mu, Nm, a3min, da3, sig3, u3, eta, nu, lm, lcc, sig1, q, cN, cO, dan, s, wavelength) + IincC(useQ ,b,s,wavelength)));
}

//calculation of absorption factor A as function of z = 2 theta [degrees]
double Calculations::A(double z, double d, double mue_ab){
    return (1. - exp( (-2. * mut(d, mue_ab) ) / (sin( (z * M_PI) / (2. * 180.) ) ) ) );
}

//exponential term for tuning of gradient/slope as function of s using parameter g
double Calculations::T(double g, double s){
    return exp(g * s);
}

//conversion factor corrAutoColl for correction of automatic collimator
//use to convert intensity for fixed colliminator to intensity for automatic collimator
//needs radius of diffractometer r, divergence angle in degrees, irradiated length l, wavelength and scattering vector s
double Calculations::corrAutoColl(double r, double delta, double l, double wavelength, double s){
    double delta_rad = (delta * M_PI / 180); //delta in radians
    return ( (r * (wavelength * s / 2) * sin(delta_rad)) / ( l * ( pow((wavelength * s / 2), 2) - pow(sin(delta_rad/2), 2) ) ) );
}


//data correction

//calculation of mut from thickness d of sample and absorption factor mue_ab
double Calculations::mut(double d, double mue_ab){
    return (d * mue_ab);
}

//calculation of muR from radius of diffractometer r and absorption factor mue_ab
double Calculations::muR(double r, double mue_ab){
    return (r * mue_ab);
}

//calculation of Dz as function of z = 2 theta [degrees]
double Calculations::Dz(double muR_, double mut_, double z){
    double degree = (M_PI / 180.);
    return ( sin(z * degree) / 2. / muR_ * (1. - 2. * mut_ / sin(z/2. * degree) / (exp(2 * mut_ / sin(z/2 * degree)) - 1)) * 180 / M_PI);
}

//calculation of intensity positions via D2thtx as function of i = position/index of intenstiy in QVector<QPointF> data
double Calculations::PositionByD2thtx(double R, double mue_ab, double t, int i){
    double x = (10. + ( 0.25 * i ) );
    double xx = ( (x/2.) * (M_PI / 180.) ); //x divided by 2 in radians
    double muet = mue_ab * t; //product of mue_ab and t
    double D2thtx = ( ( 1. / (mue_ab * M_PI * R) ) * 180. * cos(xx) * ( muet - muet * coth( muet * csc(xx) ) + sin(xx) ) );
    return  x + D2thtx;
}

//calculation of opacity for intensity correction as function of z = 2 theta [degrees]
double Calculations::opacity(double d, double mue_ab, double z){
    return ( exp( (2 * d * mue_ab) / sin( (z/2) * (M_PI / 180.) ) ) );
}

//calculation of the pragmatic approach for intensity correction as function of z = 2 theta [degrees]
double Calculations::pragmatic(double p, double z){
    return ( exp( -1. * p * pow(z,2) ) );
}


//auxiliary functions

//hyperbolic cotangent
double Calculations::coth(double x){
    return ( cosh(x) / sinh(x) );
}

//cosecant
double Calculations::csc(double x){
    return ( 1. / sin (x) );
}
