/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX’ source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "datacalculations.h"

DataCalculations::DataCalculations(QString *dataPath, QWidget *parentWidget, QObject *parent) : QObject(parent){

    this->parent = parentWidget;

    if (dataPath != 0)
        calc = new Calculations(dataPath);
    else
        calc = 0;
}

DataCalculations::~DataCalculations(){
    delete calc;
}

//converts measurement-data into the format "intensity (linear) versus 2theta [degrees]"
void DataCalculations::dataConversion(QVector<QPointF> *data, Enumerations::XFormat XFormat, Enumerations::YFormat YFormat, double wavelength){
    switch (XFormat){
        case Enumerations::twotheta_degree  :
            break;
        case Enumerations::s_oneDIVnanometer :
            for (int i = 0; i < data->size(); ++i){
                data->replace(i, QPointF(data->at(i).x() / 10, data->at(i).y()));
                data->replace(i, QPointF( asin(data->at(i).x() * wavelength / 2) / (M_PI / 180) * 2 , data->at(i).y()));
            }
            break;
        case Enumerations::s_oneDIVangstrom :
            for (int i = 0; i < data->size(); ++i)
                data->replace(i, QPointF( asin(data->at(i).x() * wavelength / 2) / (M_PI / 180) * 2 , data->at(i).y()));
            break;
        default :
            break;
    }
    switch (YFormat){
        case Enumerations::linear :
            break;
        case Enumerations::log_10 :
            for (int i = 0; i < data->size(); ++i)
                data->replace(i,QPointF(data->at(i).x(),pow(10,data->at(i).y())));
            break;
        default:
            break;
    }
}

void DataCalculations::calcDataCurve(QVector<QPointF> *data_cor, QVector<QPointF> *data, bool corrPosition, Enumerations::corrPositionMethod corrPositionMethod, bool corrIntensity, Enumerations::corrIntensityMethod corrIntensityMethod, ParameterSet *pars, double par_r, double par_d, double wavelength, QVector<QPointF> **data_for_ChooseTemplate_AutoFit, double plotMinX, double plotMaxX){
    data_cor->clear();
    for (int i = 0; i < data->size(); ++i)
        data_cor->append(QPointF(data->at(i).x(), data->at(i).y())); //copy original data

  //data correction - position
    if (corrPosition)
        switch(corrPositionMethod){
            case Enumerations::Dz :
                for (int i = 0; i < data_cor->size(); ++i)
                    data_cor->replace(i, QPointF(data_cor->at(i).x() + calc->Dz(calc->muR(par_r,pars->getParameterValue(Enumerations::mue_ab,Enumerations::val)), calc->mut(par_d,pars->getParameterValue(Enumerations::mue_ab,Enumerations::val)), data_cor->at(i).x()), data_cor->at(i).y()));
                break;
            case Enumerations::D2thtx :
                for (int i = 0; i < data_cor->size(); ++i)
                    data_cor->replace(i, QPointF(calc->PositionByD2thtx(pars->getParameterValue(Enumerations::R,Enumerations::val),pars->getParameterValue(Enumerations::mue_ab,Enumerations::val),pars->getParameterValue(Enumerations::t,Enumerations::val),i), data_cor->at(i).y()));
                break;
            default:
                break;
        }

    //data correction - intensity
    if(corrIntensity)
        switch(corrIntensityMethod){
            case Enumerations::LambertBeer: //absorption by lambert-beer
                for (int i = 0; i < data_cor->size(); ++i)
                    data_cor->replace(i,QPointF(data_cor->at(i).x(),data_cor->at(i).y() * calc->opacity(par_d,pars->getParameterValue(Enumerations::mue_ab,Enumerations::val),data_cor->at(i).x())));
                break;
            case Enumerations::Pragmatic: //pragmatic approach
                for (int i = 0; i < data_cor->size(); ++i)
                    data_cor->replace(i,QPointF(data_cor->at(i).x(),data_cor->at(i).y() * calc->pragmatic(pars->getParameterValue(Enumerations::p,Enumerations::val),data_cor->at(i).x())));
                break;
            default:
                break;
        }

    //data conversion
    for (int i = 0; i < data_cor->size(); ++i){
        data_cor->replace(i,QPointF(2 * sin(data_cor->at(i).x() / 2 * (M_PI / 180)) / wavelength, data_cor->at(i).y())); //conversion from degrees in modulus of scattering vector ( (2theta / °) to (1/s / (1/Å)) )
        data_cor->replace(i,QPointF(data_cor->at(i).x(),log10(data_cor->at(i).y()))); //logarithmic calculus (log10 of intensities)
    }

    (*data_for_ChooseTemplate_AutoFit)->clear();
    delete *data_for_ChooseTemplate_AutoFit;
    *data_for_ChooseTemplate_AutoFit = new QVector<QPointF>(*data_cor);

    for (int i = 0; i < data_cor->size(); ++i)
        if (data_cor->at(i).x() < plotMinX || data_cor->at(i).x() > plotMaxX){
            data_cor->remove(i);
            --i;
        }
}

void DataCalculations::calcFitCurve(QVector<QPointF> *target, ParameterSet *pars, double min, double max, int thinningFactor, double wavelength, bool useA, double par_d, bool useGradient, bool useCorrAutoColl, double par_r, double par_delta, double par_l, bool useQ, double par_b){
    target->clear();
    for (double s = min; s <= max; s += 0.001 * thinningFactor){
        double factA = (useA ? ( calc->A( ( ( asin( (s * wavelength / 2) ) ) * (2 * 180 / M_PI) ), par_d, pars->getParameterValue(Enumerations::mue_ab, Enumerations::val)) ) : 1);
        double factGradient = (useGradient ? calc->T(pars->getParameterValue(Enumerations::g, Enumerations::val),s) : 1);
        double factCorrAutoColl = (useCorrAutoColl ? (1. / calc->corrAutoColl(par_r,par_delta,par_l,wavelength,s)) : 1);
        target->append(QPointF(s,log10(factA * factGradient * factCorrAutoColl * calc->Ifit(useQ,par_b,pars->getParameterValue(Enumerations::k, Enumerations::val),pars->getParameterValue(Enumerations::cno, Enumerations::val),pars->getParameterValue(Enumerations::mu, Enumerations::val),pars->getParameterValue(Enumerations::mu, Enumerations::val)/pars->getParameterValue(Enumerations::beta, Enumerations::val),pars->getParameterValue(Enumerations::a3, Enumerations::val)-pars->getParameterValue(Enumerations::da3, Enumerations::val),pars->getParameterValue(Enumerations::da3, Enumerations::val),pars->getParameterValue(Enumerations::sig3, Enumerations::val),pars->getParameterValue(Enumerations::u3, Enumerations::val),pars->getParameterValue(Enumerations::eta, Enumerations::val),pars->getParameterValue(Enumerations::nu, Enumerations::val),pars->getParameterValue(Enumerations::nu, Enumerations::val)/pars->getParameterValue(Enumerations::alpha, Enumerations::val),pars->getParameterValue(Enumerations::lcc, Enumerations::val),pars->getParameterValue(Enumerations::sig1, Enumerations::val),pars->getParameterValue(Enumerations::q, Enumerations::val),pars->getParameterValue(Enumerations::cN, Enumerations::val),pars->getParameterValue(Enumerations::cO, Enumerations::val),pars->getParameterValue(Enumerations::dan, Enumerations::val),s,wavelength) + pars->getParameterValue(Enumerations::const1, Enumerations::val)) + pars->getParameterValue(Enumerations::const2, Enumerations::val)));
    }
    double s = max;
    double factA = (useA ? ( calc->A( ( ( asin( (s * wavelength / 2) ) ) * (2 * 180 / M_PI) ), par_d, pars->getParameterValue(Enumerations::mue_ab, Enumerations::val)) ) : 1);
    double factGradient = (useGradient ? calc->T(pars->getParameterValue(Enumerations::g, Enumerations::val),s) : 1);
    double factCorrAutoColl = (useCorrAutoColl ? (1. / calc->corrAutoColl(par_r,par_delta,par_l,wavelength,s)) : 1);
    target->append(QPointF(s,log10(factA * factGradient * factCorrAutoColl * calc->Ifit(useQ,par_b,pars->getParameterValue(Enumerations::k, Enumerations::val),pars->getParameterValue(Enumerations::cno, Enumerations::val),pars->getParameterValue(Enumerations::mu, Enumerations::val),pars->getParameterValue(Enumerations::mu, Enumerations::val)/pars->getParameterValue(Enumerations::beta, Enumerations::val),pars->getParameterValue(Enumerations::a3, Enumerations::val)-pars->getParameterValue(Enumerations::da3, Enumerations::val),pars->getParameterValue(Enumerations::da3, Enumerations::val),pars->getParameterValue(Enumerations::sig3, Enumerations::val),pars->getParameterValue(Enumerations::u3, Enumerations::val),pars->getParameterValue(Enumerations::eta, Enumerations::val),pars->getParameterValue(Enumerations::nu, Enumerations::val),pars->getParameterValue(Enumerations::nu, Enumerations::val)/pars->getParameterValue(Enumerations::alpha, Enumerations::val),pars->getParameterValue(Enumerations::lcc, Enumerations::val),pars->getParameterValue(Enumerations::sig1, Enumerations::val),pars->getParameterValue(Enumerations::q, Enumerations::val),pars->getParameterValue(Enumerations::cN, Enumerations::val),pars->getParameterValue(Enumerations::cO, Enumerations::val),pars->getParameterValue(Enumerations::dan, Enumerations::val),s,wavelength) + pars->getParameterValue(Enumerations::const1, Enumerations::val)) + pars->getParameterValue(Enumerations::const2, Enumerations::val)));
}

void DataCalculations::calcDeviationData(QVector<QPointF> *target, QVector<QPointF> *data, ParameterSet *pars, double wavelength, bool useA, double par_d, bool useGradient, bool useCorrAutoColl, double par_r, double par_delta, double par_l, bool useQ, double par_b){
    target->clear();
    for (int i = 0; i < data->size(); ++i){
        double intensity_exp = data->at(i).y();
        double s = data->at(i).x();
        double factA = (useA ? ( calc->A( ( ( asin( (s * wavelength / 2) ) ) * (2 * 180 / M_PI) ), par_d, pars->getParameterValue(Enumerations::mue_ab, Enumerations::val)) ) : 1);
        double factGradient = (useGradient ? calc->T(pars->getParameterValue(Enumerations::g, Enumerations::val),s) : 1);
        double factCorrAutoColl = (useCorrAutoColl ? (1. / calc->corrAutoColl(par_r,par_delta,par_l,wavelength,s)) : 1);
        double intensity_theo = log10(factA * factGradient * factCorrAutoColl * calc->Ifit(useQ,par_b,pars->getParameterValue(Enumerations::k, Enumerations::val),pars->getParameterValue(Enumerations::cno, Enumerations::val),pars->getParameterValue(Enumerations::mu, Enumerations::val),pars->getParameterValue(Enumerations::mu, Enumerations::val)/pars->getParameterValue(Enumerations::beta, Enumerations::val),pars->getParameterValue(Enumerations::a3, Enumerations::val)-pars->getParameterValue(Enumerations::da3, Enumerations::val),pars->getParameterValue(Enumerations::da3, Enumerations::val),pars->getParameterValue(Enumerations::sig3, Enumerations::val),pars->getParameterValue(Enumerations::u3, Enumerations::val),pars->getParameterValue(Enumerations::eta, Enumerations::val),pars->getParameterValue(Enumerations::nu, Enumerations::val),pars->getParameterValue(Enumerations::nu, Enumerations::val)/pars->getParameterValue(Enumerations::alpha, Enumerations::val),pars->getParameterValue(Enumerations::lcc, Enumerations::val),pars->getParameterValue(Enumerations::sig1, Enumerations::val),pars->getParameterValue(Enumerations::q, Enumerations::val),pars->getParameterValue(Enumerations::cN, Enumerations::val),pars->getParameterValue(Enumerations::cO, Enumerations::val),pars->getParameterValue(Enumerations::dan, Enumerations::val),s,wavelength) + pars->getParameterValue(Enumerations::const1, Enumerations::val)) + pars->getParameterValue(Enumerations::const2, Enumerations::val);
        target->append(QPointF(s, ((intensity_exp - intensity_theo)/intensity_theo) * 100 ));
    }
}
