/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "datafileaccess.h"

DataFileAccess::DataFileAccess(QWidget *parentWidget, QObject *parent) : QObject(parent){
    this->parent = parentWidget;
}

DataFileAccess::~DataFileAccess(){
}

bool DataFileAccess::readParsMeaning(QString *applicationDirPath, QVector<parsMean> *parsMeaning){
    bool error = false; //if any parameterMeaning cannot be read
    QFile *file = new QFile(*applicationDirPath + tr("/data/pars_de.txt"));
    if (file->open(QIODevice::ReadOnly | QIODevice::Text)){
        QTextStream *in = new QTextStream(file);
        int skipLines = 2;
        for (int i = 0; i < skipLines; ++i)
            in->readLine();
        while (!(in->atEnd())){
            QString *line = new QString(in->readLine());
            QStringList splitted = line->split(";",QString::SkipEmptyParts);
            delete line;
            parsMeaning->append(parsMean());
            int pos = parsMeaning->size() - 1;
            (*parsMeaning)[pos].symbol = splitted.at(0);
            (*parsMeaning)[pos].meaning = splitted.at(1);
        }
        file->close();
        delete in;
    }
    else
        error = true;
    delete file;
    return !error;
}

bool DataFileAccess::readParsNamesAutoFit(QString *applicationDirPath, QStringList *parsNamesAutoFit){
    bool error = false; //if any parameterMeaning cannot be read
    QFile *file = new QFile(*applicationDirPath + tr("/data/parsAutoFit_de.txt"));
    if (file->open(QIODevice::ReadOnly | QIODevice::Text)){
        QTextStream *in = new QTextStream(file);
        int skipLines = 2;
        for (int i = 0; i < skipLines; ++i)
            in->readLine();
        while (!(in->atEnd()))
            parsNamesAutoFit->append(in->readLine());
        file->close();
        delete in;
    }
    else
        error = true;
    delete file;
    return !error;
}

Enumerations::errorType DataFileAccess::loadXYFile(QVector<QPointF> *dataToRead, QString **dataPathToRead, QString *dialogCaption, QString *dataTypeName, QString *defaultDir){
    QFileDialog *dlg = new QFileDialog(parent, *dialogCaption, *defaultDir, (*dataTypeName) + " (*.txt *.dat *.xy)");
    dlg->setAcceptMode(QFileDialog::AcceptOpen);
    dlg->setFileMode(QFileDialog::ExistingFile);
    dlg->setHistory(QStringList());
    dlg->setOption(QFileDialog::ReadOnly);
    dlg->setViewMode(QFileDialog::List);
    if (dlg->exec()){
        delete *dataPathToRead;
        *dataPathToRead = new QString(dlg->selectedFiles().at(0));
        delete dlg;
        delete dataTypeName;
        delete dialogCaption;
        if (readXYFile(dataToRead, *dataPathToRead))
            return Enumerations::NoError;
        else
            return Enumerations::OtherError;
    }
    return Enumerations::DialogCancelled;
}

bool DataFileAccess::readXYFile (QVector<QPointF> *dataToRead, QString *filePath){
    bool error = false; //no error initially
    QFile *file = new QFile(*filePath);
    if(file->open(QIODevice::ReadOnly | QIODevice::Text)){
        QTextStream *in = new QTextStream(file);
        dataToRead->clear();
        while (!(in->atEnd())){
            QString *line = new QString(in->readLine());
            QStringList splitted = line->split(QRegExp("[\x09\x20\x3b]"),QString::SkipEmptyParts);
            if (splitted.size() >= 2){ //X and Y read from first two columns, additional ones are ignored
                QString *splitted0 = new QString(splitted.at(0));
                QString *splitted1 = new QString(splitted.at(1));
                if (AuxiliaryMethods::isDouble(splitted0) && AuxiliaryMethods::isDouble(splitted1))
                    dataToRead->append(QPointF(splitted0->toDouble(),splitted1->toDouble()));
                delete splitted0;
                delete splitted1;
            }
            delete line;
        }
        file->close();
        delete in;
        if (dataToRead->size() == 0)
            error = true;
    }
    else
        error = true;
    delete file;
    return !error;
}

Enumerations::errorType DataFileAccess::saveXYFile(QVector<QPointF> *dataToSave, QString *dialogCaption, QString *defaultDir, QWidget *parent){
    Enumerations::errorType error = Enumerations::NoError; //no error initially
    if (dataToSave->size() > 0){
        QFileDialog *dlg = new QFileDialog(parent, *dialogCaption, *defaultDir, tr("X-Y-Datei (*.xy)"));
        dlg->setAcceptMode(QFileDialog::AcceptSave);
        dlg->setFileMode(QFileDialog::AnyFile);
        dlg->setHistory(QStringList());
        dlg->setViewMode(QFileDialog::List);
        dlg->setDefaultSuffix("xy");
        if (dlg->exec())
            if ( !(writeXYFile(dataToSave, new QString(dlg->selectedFiles().at(0)))) )
                error = Enumerations::OtherError;
        delete dlg;
    }
    else
        error = Enumerations::OtherError;
    delete dialogCaption;
    return error;
}

bool DataFileAccess::writeXYFile (QVector<QPointF> *dataToSave, QString *filePath){
    bool error = false; //no error initially
    QFile *file = new QFile(*filePath);
    if (file->open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text)){
        QTextStream *out = new QTextStream(file);
        out->setRealNumberPrecision(12);
        for (int i = 0; i < dataToSave->size(); ++i)
            (*out) << dataToSave->at(i).x() << "\x09" << dataToSave->at(i).y() << "\n";
        out->flush();
        file->flush();
        file->close();
        delete out;
    }
    else
        error = true;
    delete file;
    delete filePath;
    return !error;
}

int DataFileAccess::readWavelengths(QString *applicationDirPath, QVector<wavelength> *wavelengths, bool skipUserFile){
    QFile *file = new QFile(*applicationDirPath + "/settings_einstellungen" + tr("/wavelengths_user_de.txt"));
    if ( !(skipUserFile) && file->open(QIODevice::ReadOnly | QIODevice::Text) )
        return readWavelengths_auxialiaryMethod(file, true, wavelengths);
    else{
        delete file;
        file = new QFile(*applicationDirPath + tr("/data/wavelengths_de.txt"));
        if (file->open(QIODevice::ReadOnly | QIODevice::Text))
           return readWavelengths_auxialiaryMethod(file, false, wavelengths);
        else{
            delete file;
            AuxiliaryMethods::errorMessage(parent, new QString(tr("Die Liste der Wellenlaengen\n\"wavelengths_de.txt\"\nkonnte nicht geladen werden.\nEs wird daher die Standard-\nWellenlaenge verwendet.")));
            wavelengths->append(wavelength());
            (*wavelengths)[wavelengths->size() - 1].name = "";
            (*wavelengths)[wavelengths->size() - 1].value = 1.5418;
            return 0;
        }
    }
}

int DataFileAccess::readWavelengths_auxialiaryMethod(QFile *file, bool userFile, QVector<wavelength> *wavelengths){
    int toSelect = 0;
    QTextStream *in = new QTextStream(file);
    int skipLines = userFile ? 5 : 2;
    for (int i = 0; i < skipLines; ++i)
        in->readLine();
    if (userFile)
        toSelect = in->readLine().toInt();
    while (!(in->atEnd())){
        QString *line = new QString(in->readLine());
        QStringList splitted = line->split(";",QString::SkipEmptyParts);
        delete line;
        wavelengths->append(wavelength());
        (splitted.at(0) == " ") ? ( (*wavelengths)[wavelengths->size() - 1].name = "" ) : ( (*wavelengths)[wavelengths->size() - 1].name = splitted.at(0) );
        (*wavelengths)[wavelengths->size() - 1].value = splitted.at(1).toDouble();
    }
    file->close();
    delete in;
    delete file;
    return toSelect;
}

void DataFileAccess::writeWavelengths(QString *applicationDirPath, int selectedWavelength, QVector<wavelength> *wavelengths){
    QFile *file = new QFile(*applicationDirPath + "/settings_einstellungen" + tr("/wavelengths_user_de.txt"));
    if (file->open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text)){
        QTextStream *out = new QTextStream(file);
        out->setRealNumberPrecision(12);
        (*out) << tr("---Diese Datei darf ab Zeile 7 manuell veraendert werden.---\n");
        (*out) << tr("---Untenstehendes Format ist unbedingt einzuhalten, sonst koennen beim Laden der Datei Abstuerze und Datenverlust auftreten!---\n");
        (*out) << tr("---Format: \"Bezeichung;Wellenlaenge in \xC5 mit Punkt als Dezimaltrennzeichen\"---\n");
        (*out) << tr("---Bezeichnung = \" \" (Leerzeichen) wird als \"keine Bezeichnung\" behandelt.---\n");
        (*out) << tr("---Soll keine Bezeichnung verwendet werden, ist das Leerzeichen zwingend anzugeben.---\n");
        (*out) << selectedWavelength << "\n";
        for (int i = 0; i < wavelengths->size(); ++i){
            (*out) << ( (wavelengths->at(i).name.length() > 0) ? (wavelengths->at(i).name) : " " );
            (*out) << ";";
            (*out) << wavelengths->at(i).value;
            (*out) << "\n";
        }
        out->flush();
        file->flush();
        file->close();
        delete out;
        delete file;
    }
    else{
        delete file;
        AuxiliaryMethods::errorMessage(parent, new QString(tr("Die benutzerdefinierten Wellenlaengen\nkonnten nicht gespeichert werden.")));
    }
}

bool DataFileAccess::saveImageFile(QPixmap *imageDataToWrite, QString *dialogCaption, QString *defaultDir, QWidget *parent){
    bool error = false; //no error initially
    QFileDialog *dlg = new QFileDialog(parent,*dialogCaption,*defaultDir,tr("Bitmap (*.bmp);;JPEG-Datei (*.jpg);;PNG-Datei (*.png)"));
    dlg->setAcceptMode(QFileDialog::AcceptSave);
    dlg->setFileMode(QFileDialog::AnyFile);
    dlg->setHistory(QStringList());
    dlg->setViewMode(QFileDialog::List);
    if (dlg->exec()){
        switch (dlg->selectedNameFilter().at(0).toLatin1()){
            case 'B' :
                if (!imageDataToWrite->save(dlg->selectedFiles().at(0), "BMP" , -1))
                    error = true;
                break;
            case 'J' :
                if (!imageDataToWrite->save(dlg->selectedFiles().at(0), "JPG" , -1))
                    error = true;
                break;
            case 'P' :
                if (!imageDataToWrite->save(dlg->selectedFiles().at(0), "PNG" , -1))
                    error = true;
                break;
            default:
                error = true;
                break;
        }
    }
    delete dlg;
    delete dialogCaption;
    return !error;
}
