/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef DATAFILEACCESS_H
#define DATAFILEACCESS_H

#include <QFile>
#include <QFileDialog>
#include <QIODevice>
#include <QPixmap>
#include <QPointF>
#include <QString>
#include <QStringList>
#include <QTextStream>
#include <QVector>

#include "auxiliarymethods.h"
#include "enumerations.h"
#include "structs.h"

#include <QObject>

class DataFileAccess : public QObject{

    Q_OBJECT

public:
    explicit DataFileAccess(QWidget *parentWidget = 0, QObject *parent = 0);
    ~DataFileAccess();

    bool readParsMeaning(QString *applicationDirPath, QVector<parsMean> *parsMeaning);

    bool readParsNamesAutoFit(QString *applicationDirPath, QStringList *parsNamesAutoFit);


    Enumerations::errorType loadXYFile(QVector<QPointF> *dataToRead, QString **dataPathToRead, QString *dialogCaption, QString *dataTypeName, QString *defaultDir);
        //QStrings except dataPathToRead and defaultDir will be deleted
    bool readXYFile (QVector<QPointF> *dataToRead, QString *filePath);


    Enumerations::errorType saveXYFile(QVector<QPointF> *dataToSave, QString *dialogCaption, QString *defaultDir, QWidget *parent);
        //QString dialogCaption will be deleted
    bool writeXYFile (QVector<QPointF> *dataToSave, QString *filePath);
        //QStrings will be deleted


    //read wavelength-structures, returns index of wavelength-structure chosen on former exit
    int readWavelengths(QString *applicationDirPath, QVector<wavelength> *wavelengths, bool skipUserFile);

    //save wavelength-structuress on exit into user-file
    void writeWavelengths(QString *applicationDirPath, int selectedWavelength, QVector<wavelength> *wavelengths);


    //save QPixmap to image file, dialogCaption will be deleted
    bool saveImageFile(QPixmap *imageDataToWrite, QString *dialogCaption, QString *defaultDir, QWidget *parent);


private:
    QWidget *parent;

    //auxialary method for readWavelengths
    int readWavelengths_auxialiaryMethod(QFile *file, bool userFile, QVector<wavelength> *wavelengths);

};

#endif // DATAFILEACCESS_H
