/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef DIALOG_CHOOSETEMPLATE_H
#define DIALOG_CHOOSETEMPLATE_H

#include <QBrush>
#include <QCloseEvent>
#include <QColor>
#include <QFont>
#include <QKeyEvent>
#include <QLabel>
#include <QPen>
#include <QPointF>
#include <QSize>
#include <QString>
#include <QVector>

#include <qwt_curve_fitter.h>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include <qwt_series_data.h>
#include <qwt_plot_grid.h>
#include <qwt_symbol.h>

#include "datacalculations.h"
#include "datastorage.h"
#include "parameterfileaccess.h"
#include "settingsstorage.h"

#include <QDialog>

namespace Ui {
    class Dialog_chooseTemplate;
}

class Dialog_chooseTemplate : public QDialog{

    Q_OBJECT

public:
    explicit Dialog_chooseTemplate(DataCalculations *datacal = 0, DataStorage *dst = 0, SettingsStorage *sst = 0, ParameterFileAccess *pfa = 0, QVector<QPointF> *data = 0, QWidget *parent = 0);
    ~Dialog_chooseTemplate();

private:
    Ui::Dialog_chooseTemplate *ui;

    DataCalculations *datacal;
    DataStorage *dst;
    SettingsStorage *sst;
    ParameterFileAccess *pfa;
    QVector<QPointF> *data;

    QColor *colorData;
    QBrush *brushData;
    QSize *sizeData;
    QPen *penData;
    QwtSymbol *symbolData;
    QwtPlotCurve *curveData;

    QColor *colorTemplate;
    QPen *penTemplate;
    QwtPlotCurve *curveTemplate[DataStorage::number_of_templates];
    QwtPlot *plotTemplate[DataStorage::number_of_templates];
    QLabel *labelTemplate[DataStorage::number_of_templates];

    //close allowed? (close only by click on OK)
    bool canClose;

    //Events
    void keyPressEvent(QKeyEvent *event);
    void closeEvent(QCloseEvent *event);

private slots:

    void on_button_chooseTemplate_clicked();
    void on_button_chooseNoTemplate_clicked();
    void on_button_calculateTemplates_clicked();

};

#endif // DIALOG_CHOOSETEMPLATE_H
