/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "dialog_preferences.h"
#include "ui_dialog_preferences.h"

Dialog_Preferences::Dialog_Preferences(ProgramSettingsSet *prgset, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Dialog_Preferences){

    ui->setupUi(this);

    //own code

    canClose = false;

    this->prgset = prgset;

    this->setFixedSize(this->width(), this->height());
    this->setWindowFlags(Qt::MSWindowsFixedSizeDialogHint);
    this->setWindowFlags(Qt::CustomizeWindowHint);
    setFocus();

    ui->plot_thicknessPreview->enableAxis(QwtPlot::xBottom,false);
    ui->plot_thicknessPreview->enableAxis(QwtPlot::yLeft,false);
    ui->plot_thicknessPreview->setAxisAutoScale(QwtPlot::xBottom,false);
    ui->plot_thicknessPreview->setAxisAutoScale(QwtPlot::yLeft,false);
    ui->plot_thicknessPreview->setAxisScale(QwtPlot::xBottom,0,15,2);
    ui->plot_thicknessPreview->setAxisScale(QwtPlot::yLeft,0,15,2);

    dots = new QVector<QPointF>(0);
    line = new QVector<QPointF>(0);
    for (int i = 0; i <= 15; ++i){
        dots->append(QPointF(i,i));
        line->append(QPointF(i,i));
    }

    grid = new QwtPlotGrid();
    grid->enableX(true);
    grid->enableY(true);
    grid->attach(ui->plot_thicknessPreview);

    curveDots = new QwtPlotCurve("dots");
    curveDots->setStyle(QwtPlotCurve::Dots);
    colorDots = new QColor(Qt::blue);
    brushDots = new QBrush (*colorDots);
    penDots = new QPen(*colorDots);
    sizeDots = new QSize(prgset->thicknessOfCurveDots, prgset->thicknessOfCurveDots);
    symbolDots = new QwtSymbol(QwtSymbol::Ellipse, *brushDots, *penDots, *sizeDots);
    curveDots->setSymbol(symbolDots);
    curveDots->setData(new QwtPointSeriesData(*dots));

    curveLine = new QwtPlotCurve("line");
    curveLine->setStyle(QwtPlotCurve::Lines);
    colorLine = new QColor(Qt::red);
    penLine = new QPen(*colorLine);
    penLine->setWidth(prgset->thicknessOfCurveLines);
    curveLine->setPen(*penLine);
    curveLine->setCurveFitter(new QwtSplineCurveFitter());
    curveLine->setData(new QwtPointSeriesData(*line));

    setWidgetValues();

}


Dialog_Preferences::~Dialog_Preferences(){

    curveLine->detach();
    delete curveLine;
    delete penLine;
    delete colorLine;

    curveDots->detach();
    delete curveDots;
    delete sizeDots;
    delete penDots;
    delete brushDots;
    delete colorDots;

    grid->detach();
    delete grid;

    line->clear();
    delete line;

    dots->clear();
    delete dots;

    delete ui;
}


void Dialog_Preferences::swapDotsLine(Enumerations::drawingOrder order){
    curveDots->detach();
    curveLine->detach();
    switch(order){
        case Enumerations::dotsInFront :
            curveLine->attach(ui->plot_thicknessPreview);
            curveDots->attach(ui->plot_thicknessPreview);
            break;
        case Enumerations::lineInFront :
            curveDots->attach(ui->plot_thicknessPreview);
            curveLine->attach(ui->plot_thicknessPreview);
            break;
        default:
            curveLine->attach(ui->plot_thicknessPreview);
            curveDots->attach(ui->plot_thicknessPreview);
            break;
    }
}

void Dialog_Preferences::setWidgetValues(){
    ui->edit_defaultDir->setText(*(prgset->defaultDir));
    ui->checkBox_fullPath->setChecked(prgset->displayFullPath);
    ui->checkBox_adjustBorders->setChecked(prgset->adjustAxisIntervals);
    ui->spinBox_thicknessOfCurveDots->setValue(prgset->thicknessOfCurveDots);
    ui->spinBox_thicknessOfCurveLines->setValue(prgset->thicknessOfCurveLines);
    ui->comboBox_elementInFront->setCurrentIndex(prgset->elementInFront);
    ui->checkBox_enableChooseTemplate->setChecked(prgset->enableChooseTemplate);
    ui->checkBox_calcTemplates->setChecked(prgset->calcTemplates);
    ui->edit_kernelPath->setText(*(prgset->kernelPath));

    swapDotsLine(Enumerations::drawingOrder(ui->comboBox_elementInFront->currentIndex()));
}


void Dialog_Preferences::keyPressEvent(QKeyEvent *event){
    event->accept();
}


void Dialog_Preferences::closeEvent(QCloseEvent *event){
    if (canClose)
        event->accept();
    else
        event->ignore();
}


void Dialog_Preferences::on_edit_defaultDir_editingFinished(){
    bool prevState = ui->edit_defaultDir->blockSignals(true); //save previous state
    QDir *checkDir = new QDir();
    if (!(checkDir->exists(ui->edit_defaultDir->text())))
        ui->edit_defaultDir->setText(*(prgset->defaultDir));
    delete checkDir;
    ui->edit_defaultDir->blockSignals(prevState); //restore previous state
}


void Dialog_Preferences::on_button_choseDefaultDir_clicked(){
    QFileDialog *dlg = new QFileDialog(this,tr("Standardverzeichnis waehlen"),ui->edit_defaultDir->text(),tr("Verzeichnisse"));
    dlg->setAcceptMode(QFileDialog::AcceptOpen);
    dlg->setFileMode(QFileDialog::Directory);
    dlg->setHistory(QStringList());
    dlg->setOption(QFileDialog::ReadOnly);
    dlg->setOption(QFileDialog::ShowDirsOnly);
    dlg->setViewMode(QFileDialog::List);
    if (dlg->exec() && QDir(dlg->selectedFiles().at(0)).exists()){
        ui->edit_defaultDir->setText(dlg->selectedFiles().at(0));
    }
    delete dlg;
}


void Dialog_Preferences::on_spinBox_thicknessOfCurveDots_valueChanged(int arg1){
    delete sizeDots;
    sizeDots = new QSize(arg1,arg1);
    symbolDots = new QwtSymbol(QwtSymbol::Ellipse, *brushDots, *penDots, *sizeDots);
    curveDots->setSymbol(symbolDots);
    ui->plot_thicknessPreview->replot();
}


void Dialog_Preferences::on_spinBox_thicknessOfCurveLines_valueChanged(int arg1){
    delete penLine;
    penLine = new QPen(*colorLine);
    penLine->setWidth(arg1);
    curveLine->setPen(*(penLine));
    ui->plot_thicknessPreview->replot();
}


void Dialog_Preferences::on_button_swapColors_clicked(){

    if (curveDots->symbol()->pen().color() == Qt::blue && curveLine->pen().color() == Qt::red){
        delete penDots;
        delete brushDots;
        delete colorDots;
        colorDots = new QColor(Qt::darkGreen);
        brushDots = new QBrush (*colorDots);
        penDots = new QPen(*colorDots);
        symbolDots = new QwtSymbol(QwtSymbol::Ellipse, *brushDots, *penDots, *sizeDots);
        curveDots->setSymbol(symbolDots);

        delete penLine;
        delete colorLine;
        colorLine = new QColor(Qt::yellow);
        penLine = new QPen(*colorLine);
        penLine->setWidth(ui->spinBox_thicknessOfCurveLines->value());
        curveLine->setPen(*penLine);
    }
    else{
        delete penDots;
        delete brushDots;
        delete colorDots;
        colorDots = new QColor(Qt::blue);
        brushDots = new QBrush (*colorDots);
        penDots = new QPen(*colorDots);
        symbolDots = new QwtSymbol(QwtSymbol::Ellipse, *brushDots, *penDots, *sizeDots);
        curveDots->setSymbol(symbolDots);
        delete penLine;
        delete colorLine;
        colorLine = new QColor(Qt::red);
        penLine = new QPen(*colorLine);
        penLine->setWidth(ui->spinBox_thicknessOfCurveLines->value());
        curveLine->setPen(*penLine);
    }

    ui->plot_thicknessPreview->replot();
}


void Dialog_Preferences::on_comboBox_elementInFront_currentIndexChanged(int index){
    swapDotsLine(Enumerations::drawingOrder(index));
    ui->plot_thicknessPreview->replot();
}


void Dialog_Preferences::on_edit_kernelPath_editingFinished(){
    bool prevState = ui->edit_kernelPath->blockSignals(true); //save previous state
    QDir *checkDir = new QDir();
    if (!(checkDir->exists(ui->edit_kernelPath->text())))
        ui->edit_kernelPath->setText(*(prgset->kernelPath));
    delete checkDir;
    ui->edit_kernelPath->blockSignals(prevState); //restore previous state
}

void Dialog_Preferences::on_button_choseKernelPath_clicked(){
    QFileDialog *dlg = new QFileDialog(this,tr("Pfad des Mathematica-Kernels waehlen"), ui->edit_kernelPath->text(), tr("(*.exe)"));
    dlg->setAcceptMode(QFileDialog::AcceptOpen);
    dlg->setFileMode(QFileDialog::ExistingFile);
    dlg->setHistory(QStringList());
    dlg->setOption(QFileDialog::ReadOnly);
    dlg->setViewMode(QFileDialog::List);
    if (dlg->exec() && QFile(dlg->selectedFiles().at(0)).exists()){
        ui->edit_kernelPath->setText(dlg->selectedFiles().at(0));
    }
    delete dlg;
}

void Dialog_Preferences::on_button_OK_clicked(){
    delete prgset->defaultDir;
    prgset->defaultDir = new QString(ui->edit_defaultDir->text());
    prgset->displayFullPath = ui->checkBox_fullPath->isChecked();
    prgset->adjustAxisIntervals = ui->checkBox_adjustBorders->isChecked();
    prgset->thicknessOfCurveDots = ui->spinBox_thicknessOfCurveDots->value();
    prgset->thicknessOfCurveLines = ui->spinBox_thicknessOfCurveLines->value();
    prgset->elementInFront = Enumerations::drawingOrder(ui->comboBox_elementInFront->currentIndex());
    prgset->enableChooseTemplate = ui->checkBox_enableChooseTemplate->isChecked();
    prgset->calcTemplates = ui->checkBox_calcTemplates->isChecked();
    delete prgset->kernelPath;
    prgset->kernelPath = new QString(ui->edit_kernelPath->text());
    setResult(QDialog::Accepted);
    canClose = true;
    close();
}


void Dialog_Preferences::on_button_Cancel_clicked(){
    setResult(QDialog::Rejected);
    canClose = true;
    close();
}


void Dialog_Preferences::on_button_defaultValues_clicked(){
    ui->edit_defaultDir->setText(prgset->get_defaultDir_default());
    ui->checkBox_fullPath->setChecked(prgset->get_displayFullPath_default());
    ui->checkBox_adjustBorders->setChecked(prgset->get_adjustAxisIntervals_default());
    ui->spinBox_thicknessOfCurveDots->setValue(prgset->get_thicknessOfCurveDots_default());
    ui->spinBox_thicknessOfCurveLines->setValue(prgset->get_thicknessOfCurveLines_default());
    ui->comboBox_elementInFront->setCurrentIndex(prgset->get_elementInFront_default());
    ui->checkBox_enableChooseTemplate->setChecked(prgset->get_enableChooseTemplate_default());
    ui->checkBox_calcTemplates->setChecked(prgset->get_calcTemplates_default());
    ui->edit_kernelPath->setText(prgset->get_kernelPath_default());
}
