/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "parameterfileaccess.h"

ParameterFileAccess::ParameterFileAccess(DataCalculations *datacal, QWidget *parentWidget, QObject *parent) : QObject(parent){
    this->parent = parentWidget;
    this->datacal = datacal;
}



ParameterFileAccess::~ParameterFileAccess(){

}



Enumerations::errorType ParameterFileAccess::loadPars (DataStorage *dst, SettingsStorage *sst){
    Enumerations::errorType error = Enumerations::NoError; //no error initially
    QFileDialog *dlg = new QFileDialog(parent,tr("Parameter oeffnen"),*(sst->prgset->defaultDir),tr("Parameter (*.ngc)"));
    dlg->setAcceptMode(QFileDialog::AcceptOpen);
    dlg->setFileMode(QFileDialog::ExistingFile);
    dlg->setHistory(QStringList());
    dlg->setOption(QFileDialog::ReadOnly);
    dlg->setViewMode(QFileDialog::List);
    if (dlg->exec()){
        if (readPars(new QString(dlg->selectedFiles().at(0)), dst, sst)){
            delete dst->parsPath;
            dst->parsPath = new QString(dlg->selectedFiles().at(0));
        }
        else error = Enumerations::OtherError;
    }
    else
        error = Enumerations::DialogCancelled;
    delete dlg;
    return error;
}



bool ParameterFileAccess::readPars (QString *readPath, DataStorage *dst, SettingsStorage *sst){

    bool error = false; //no error initially

    error = !(readSliderPars(new QString (*readPath), dst));

    QVariant tempVariant; //for temporary data

    QSettings *settings = new QSettings(*readPath, QSettings::IniFormat);

    settings->sync();

    //BEGIN DO NOT TRANSLATE


    settings->beginGroup("figure-settings");

    tempVariant = settings->value("drawData");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        sst->drawData = tempVariant.toBool();

    tempVariant = settings->value("drawFit");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        sst->drawFit = tempVariant.toBool();

    tempVariant = settings->value("thinning-factor");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Int))
        dst->prgpar->thinningFactor = tempVariant.toInt();

    tempVariant = settings->value("plotMinX");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
        dst->prgpar->plotMinX = tempVariant.toDouble();

    tempVariant = settings->value("plotMaxX");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
        dst->prgpar->plotMaxX = tempVariant.toDouble();

    tempVariant = settings->value("autoscale_Y-axis");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        dst->prgpar->autoScaleYAxis = tempVariant.toBool();

    tempVariant = settings->value("plotMinY");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
        dst->prgpar->plotMinY = tempVariant.toDouble();

    tempVariant = settings->value("plotMaxY");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
        dst->prgpar->plotMaxY = tempVariant.toDouble();

    settings->endGroup();


    settings->beginGroup("measurement-parameters");

    tempVariant = settings->value("diffractometer-radius");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
        dst->prgpar->par_r = tempVariant.toDouble();

    tempVariant = settings->value("sample-thickness");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
        dst->prgpar->par_d = tempVariant.toDouble();

    tempVariant = settings->value("pass-band");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
        dst->prgpar->par_b = tempVariant.toDouble();

    tempVariant = settings->value("divergence-angle");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
        dst->prgpar->par_delta = tempVariant.toDouble();

    tempVariant = settings->value("irradiated-length");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
        dst->prgpar->par_l = tempVariant.toDouble();

    tempVariant = settings->value("wavelength");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::String)){
        QString *temp = new QString(tempVariant.toString());
        wavelength *tempWavelength = AuxiliaryMethods::validateAndParse_Wavelength(temp);
        if (tempWavelength != 0){
            delete dst->prgpar->currentWavelength;
            dst->prgpar->currentWavelength = tempWavelength;
        }
    }

    settings->endGroup();


    settings->beginGroup("fit-options");

    tempVariant = settings->value("useQ");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        dst->prgpar->useQ = tempVariant.toBool();

    tempVariant = settings->value("useA");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        dst->prgpar->useA = tempVariant.toBool();

    tempVariant = settings->value("reCheck_optA");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        sst->reCheck_optA_MainWindow = tempVariant.toBool();

    tempVariant = settings->value("useGradient");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
       dst->prgpar-> useGradient = tempVariant.toBool();

    tempVariant = settings->value("useCorrAutoColl");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        dst->prgpar->useCorrAutoColl = tempVariant.toBool();

    settings->endGroup();


    settings->beginGroup("data-general");

    tempVariant = settings->value("data-path");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::String)){
        delete dst->dataPath;
        dst->dataPath = new QString(tempVariant.toString());
        if (dst->dataPath->isEmpty()){
            delete dst->dataPath;
            dst->dataPath = 0;
        }
    }

    settings->endGroup();


    settings->beginGroup("data-conversion");

    tempVariant = settings->value("XFormat");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Int))
        dst->prgpar->XFormat = Enumerations::XFormat(tempVariant.toInt());

    tempVariant = settings->value("YFormat");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Int))
        dst->prgpar->YFormat = Enumerations::YFormat(tempVariant.toInt());

    settings->endGroup();


    settings->beginGroup("data-correction");

    tempVariant = settings->value("corrPosition");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        dst->prgpar->corrPosition = tempVariant.toBool();

    tempVariant = settings->value("corrPositionMethod");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Int))
        dst->prgpar->corrPositionMethod = Enumerations::corrPositionMethod(tempVariant.toInt());

    tempVariant = settings->value("corrIntensity");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        dst->prgpar->corrIntensity = tempVariant.toBool();

    tempVariant = settings->value("corrIntensityMethod");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Int))
        dst->prgpar->corrIntensityMethod = Enumerations::corrIntensityMethod(tempVariant.toInt());

    tempVariant = settings->value("reCheck_optIntensity");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        sst->reCheck_optIntensity_MainWindow = tempVariant.toBool();

    settings->endGroup();


    settings->beginGroup("program-settings");

    tempVariant = settings->value("scrollBarControls-value_plus_visibleSliders");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Int))
        sst->scrollBarControls_Value_temp = (tempVariant.toInt() - sst->visibleSlider_MainWindow);

    tempVariant = settings->value("drawDev");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        sst->drawDev =  tempVariant.toBool();

    tempVariant = settings->value("reCheck_buttonShowDeviation");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
         sst->reCheck_buttonShowDeviation_MainWindow = tempVariant.toBool();

    tempVariant = settings->value("dev-Interpolation");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        dst->prgpar->showInterpolation = tempVariant.toBool();

    tempVariant = settings->value("dev-FollowChanges");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
         dst->prgpar->followChanges = tempVariant.toBool();

    settings->endGroup();

    //END DO NOT TRANSLATE

    if ( ! ( (settings->status() == QSettings::NoError) && (QFile::exists(*readPath) ) ) )
        error = true;

    delete settings;

    delete readPath;

    return !error;

}



bool ParameterFileAccess::readSliderPars (QString *readPath, DataStorage *dst){

    bool error = false; //no error initially

    QVariant tempVariant; //for temporary data

    QSettings *settings = new QSettings(*readPath, QSettings::IniFormat);

    settings->sync();

    for (int i = 0; i < Enumerations::countOf_parName; ++i){
        settings->beginGroup(dst->parsMeaning->at(i).symbol);
        for (int j = 0; j < Enumerations::countOf_parType; ++j){
            tempVariant = settings->value(dst->parsType->at(j));
            if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
                dst->pars->setParameterValue(Enumerations::parName(i), Enumerations::parType(j), tempVariant.toDouble(), false);
        }
        settings->endGroup();
    }

    if ( ! ( (settings->status() == QSettings::NoError) && (QFile::exists(*readPath) ) ) )
        error = true;

    delete settings;

    delete readPath;

    return !error;
}



bool ParameterFileAccess::savePars(DataStorage *dst, SettingsStorage *sst, int scrollBarControls_value){
    bool error = false; //no error initially
    QFileDialog *dlg = new QFileDialog(parent,tr("Parameter speichern"),*(sst->prgset->defaultDir),tr("Parameter (*.ngc)"));
    dlg->setAcceptMode(QFileDialog::AcceptSave);
    dlg->setFileMode(QFileDialog::AnyFile);
    dlg->setHistory(QStringList());
    dlg->setViewMode(QFileDialog::List);
    dlg->setDefaultSuffix("ngc");
    if (dlg->exec())
        if (!writePars(new QString(dlg->selectedFiles().at(0)), dst, sst, scrollBarControls_value))
            error = true;
    delete dlg;
    return !error;
}



bool ParameterFileAccess::writePars(QString *writePath, DataStorage *dst, SettingsStorage *sst, int scrollBarControls_value){
    bool error = false; //no error initially

    QSettings *settings = new QSettings(*writePath, QSettings::IniFormat);

    settings->clear();
    settings->sync();

    settings->beginGroup(tr("WARNUNG"));
    settings->setValue(tr("WARNUNG1"), tr("---Diese Datei darf nicht manuell veraendert werden!---"));
    settings->setValue(tr("WARNUNG2"), tr("---Manuelle Aenderungen koennen beim Laden der Datei Abstuerze und Datenverlust verursachen!---"));
    settings->endGroup();

    settings->beginGroup(tr("Parameter-Beginn"));
    settings->setValue(tr("HINWEIS"), tr("Nach diesem Hinweis folgen die Werte der Anpassungs-Parameter mit Schieberegler mit je einer Gruppe fuer jeden Parameter."));
    settings->endGroup();


    for (int i = 0; i < Enumerations::countOf_parName; ++i){
        settings->beginGroup(dst->parsMeaning->at(i).symbol);
        for (int j = 0; j < Enumerations::countOf_parType; ++j)
            settings->setValue(dst->parsType->at(j), dst->pars->getParameterValue(Enumerations::parName(i), Enumerations::parType(j)));
        settings->endGroup();
    }

    settings->beginGroup(tr("Parameter-Ende"));
    settings->setValue(tr("HINWEIS"), tr("Nach diesem Hinweis folgen alle anderen Parameter in entsprechenden Gruppen."));
    settings->endGroup();

    //BEGIN DO NOT TRANSLATE

    settings->beginGroup("figure-settings");
    settings->setValue("drawData", sst->drawData);
    settings->setValue("drawFit", sst->drawFit);
    settings->setValue("thinning-factor", dst->prgpar->thinningFactor);
    settings->setValue("plotMinX",dst-> prgpar->plotMinX);
    settings->setValue("plotMaxX", dst->prgpar->plotMaxX);
    settings->setValue("autoscale_Y-axis", dst->prgpar->autoScaleYAxis);
    settings->setValue("plotMinY", dst->prgpar->plotMinY);
    settings->setValue("plotMaxY", dst->prgpar->plotMaxY);
    settings->endGroup();

    settings->beginGroup("measurement-parameters");
    settings->setValue("diffractometer-radius", dst->prgpar->par_r);
    settings->setValue("sample-thickness", dst->prgpar->par_d);
    settings->setValue("pass-band", dst->prgpar->par_b);
    settings->setValue("divergence-angle", dst->prgpar->par_delta);
    settings->setValue("irradiated-length", dst->prgpar->par_l);
    settings->setValue("wavelength", ( ( dst->prgpar->currentWavelength->name.length() > 0) ? ( dst->prgpar->currentWavelength->name + " - ") : "" ) + QString::number( dst->prgpar->currentWavelength->value,'g',10));
    settings->endGroup();

    settings->beginGroup("fit-options");
    settings->setValue("useQ", dst->prgpar->useQ);
    settings->setValue("useA", dst->prgpar->useA);
    settings->setValue("reCheck_optA",  sst->reCheck_optA_MainWindow);
    settings->setValue("useGradient", dst->prgpar->useGradient);
    settings->setValue("useCorrAutoColl", dst->prgpar->useCorrAutoColl);
    settings->endGroup();

    settings->beginGroup("data-general");
    if (dst->dataPath != 0)
        settings->setValue("data-path", *(dst->dataPath));
    else
        settings->setValue("data-path", "");
    settings->endGroup();

    settings->beginGroup("data-conversion");
    settings->setValue("XFormat", dst->prgpar->XFormat);
    settings->setValue("YFormat", dst->prgpar->YFormat);
    settings->endGroup();

    settings->beginGroup("data-correction");
    settings->setValue("corrPosition", dst->prgpar->corrPosition);
    settings->setValue("corrPositionMethod", dst->prgpar->corrPositionMethod);
    settings->setValue("corrIntensity", dst->prgpar->corrIntensity);
    settings->setValue("corrIntensityMethod", dst->prgpar->corrIntensityMethod);
    settings->setValue("reCheck_optIntensity", sst->reCheck_optIntensity_MainWindow);
    settings->endGroup();

    settings->beginGroup("program-settings");
    settings->setValue("scrollBarControls-value_plus_visibleSliders", (scrollBarControls_value + sst->visibleSlider_MainWindow) );
    settings->setValue("drawDev", sst->drawDev);
    settings->setValue("reCheck_buttonShowDeviation",  sst->reCheck_buttonShowDeviation_MainWindow);
    settings->setValue("dev-Interpolation", dst->prgpar->showInterpolation);
    settings->setValue("dev-FollowChanges", dst->prgpar->followChanges);
    settings->endGroup();

    //END DO NOT TRANSLATE

    settings->sync();

    if ( !(settings->isWritable() && (settings->status() == QSettings::NoError) ) )
        error = true;

    delete settings;

    if (!error){
        delete dst->parsPath;
        dst->parsPath = writePath;
        sst->mustSave = false;
    }
    else
        delete writePath;

    return !error;
}



bool ParameterFileAccess::readParsTPL(ParameterSet *target, bool &useA, bool &useGradient, QString **templateName, QString *readPath, DataStorage *dst, bool setDefaultValues){

    bool error = false; //no error initially

    if (setDefaultValues)
        dst->readParsDefaultValues(target); //set default values first
    *templateName = new QString(); //clear template name

    //load template

    QVariant tempVariant; //for temporary data

    QSettings *settings = new QSettings(*readPath, QSettings::IniFormat);

    settings->sync();

    for (int i = 0; i <= Enumerations::mue_ab; ++i){
        settings->beginGroup(dst->parsMeaning->at(i).symbol);
        for (int j = 0; j < Enumerations::countOf_parType; ++j){
            tempVariant = settings->value(dst->parsType->at(j));
            if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
                target->setParameterValue(Enumerations::parName(i), Enumerations::parType(j), tempVariant.toDouble(), false);
        }
        settings->endGroup();
    }

    //BEGIN DO NOT TRANSLATE

    settings->beginGroup("fit-options");

    tempVariant = settings->value("useA");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        useA = tempVariant.toBool();

    tempVariant = settings->value("useGradient");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        useGradient = tempVariant.toBool();

    settings->endGroup();

    settings->beginGroup("template-name");

    tempVariant = settings->value("name");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::String)){
        delete *templateName;
        *templateName = new QString(tempVariant.toString());
    }
    else{
        QStringList splitted = readPath->split("/");
        delete *templateName;
        *templateName = new QString(splitted.at(splitted.size() - 1));
        (*templateName)->chop(4);
    }

    settings->endGroup();

    //END DO NOT TRANSLATE

    if ( ! ( (settings->status() == QSettings::NoError) && (QFile::exists(*readPath) ) ) )
        error = true;

    delete settings;

    delete readPath;

    return !error;
}



bool ParameterFileAccess::readXYTPL(int index, double &wavelength_value, double &sample_thickness, QString *readPath, DataStorage *dst){

    bool error = false; //no error initially

    //for temporary data
    QVariant tempVariant;
    QVariant tempVariantX;
    QVariant tempVariantY;

    QSettings *settings = new QSettings(*readPath, QSettings::IniFormat);

    settings->sync();

    //BEGIN DO NOT TRANSLATE

    settings->beginGroup("X-Y");

    tempVariant = settings->value("wavelength-value");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
        wavelength_value = tempVariant.toDouble();

    tempVariant = settings->value("sample-thickness");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
        sample_thickness = tempVariant.toDouble();

    double min = 0.01;
    double max = (floor(( (2 * sin(70 * M_PI / 180)) / (wavelength_value) ) * 10.) / 10.);

    for (double s = min; s <= max; s += 0.001){
        tempVariantX = settings->value("X_" + QString::number(s, 'f', 3));
        tempVariantY = settings->value("Y_" + QString::number(s, 'f', 3));
        if (tempVariantX.isValid() && tempVariantX.canConvert(QVariant::Double) && tempVariantY.isValid() && tempVariantY.canConvert(QVariant::Double))
            dst->fitTemplate[index]->append(QPointF(tempVariantX.toDouble(), tempVariantY.toDouble()));
    }
    double s = max;
    tempVariantX = settings->value("X_" + QString::number(s, 'f', 3));
    tempVariantY = settings->value("Y_" + QString::number(s, 'f', 3));
    if (tempVariantX.isValid() && tempVariantX.canConvert(QVariant::Double) && tempVariantY.isValid() && tempVariantY.canConvert(QVariant::Double))
        dst->fitTemplate[index]->append(QPointF(tempVariantX.toDouble(), tempVariantY.toDouble()));

    settings->endGroup();

    //END DO NOT TRANSLATE

    if (dst->fitTemplate[index]->length() <= 0) //X-Y-data could not be loaded
        error = true;

    if ( ! ( (settings->status() == QSettings::NoError) && (QFile::exists(*readPath) ) ) )
        error = true;

    delete settings;

    delete readPath;

    return !error;
}



bool ParameterFileAccess::saveParsTPL(DataStorage *dst, QString *defaultDir){
    bool error = false; //no error initially
    QString *templateName;
    QString *templatePath;
    if (dst->parsPath != 0){
        templatePath = new QString(*(dst->parsPath));
        templatePath->truncate(templatePath->lastIndexOf("."));
        QStringList splitted = templatePath->split("/");
        templateName = new QString(splitted.at(splitted.size() - 1));
    }
    else
        if (dst->dataPath != 0){
            templatePath = new QString(*(dst->dataPath));
            templatePath->truncate(templatePath->lastIndexOf("."));
            QStringList splitted = templatePath->split("/");
            templateName = new QString(splitted.at(splitted.size() - 1));
        }
        else{
            templatePath = new QString(*defaultDir + "/");
            templateName = new QString();
        }

    QFileDialog *saveDlg = new QFileDialog(parent,tr("Vorlage speichern"), *templatePath, tr("Vorlage (*.tpl)"));
    saveDlg->setAcceptMode(QFileDialog::AcceptSave);
    saveDlg->setFileMode(QFileDialog::AnyFile);
    saveDlg->setViewMode(QFileDialog::List);
    saveDlg->setDefaultSuffix("tpl");
    if (saveDlg->exec()){
        Dialog_name *dlg = new Dialog_name(new QString(tr("Name der Vorlage eingeben")), templateName);
        dlg->exec();
        if (!writeParsTPL(new QString(saveDlg->selectedFiles().at(0)), dlg->getName(), dst))
            error = true;
        delete dlg;
    }
    delete saveDlg;
    delete templatePath;
    return !error;
}



bool ParameterFileAccess::writeParsTPL(QString *writePath, QString *templateName, DataStorage *dst){

    bool error = false; //no error initially

    QSettings *settings = new QSettings(*writePath, QSettings::IniFormat);

    settings->clear();
    settings->sync();

    settings->beginGroup(tr("WARNUNG"));
    settings->setValue(tr("WARNUNG1"), tr("---Diese Datei darf nicht manuell veraendert werden!---"));
    settings->setValue(tr("WARNUNG2"), tr("---Manuelle Aenderungen koennen beim Laden der Datei Abstuerze und Datenverlust verursachen!---"));
    settings->endGroup();

    settings->beginGroup(tr("Parameter-Beginn"));
    settings->setValue(tr("HINWEIS"), tr("Nach diesem Hinweis folgen die Werte der Anpassungs-Parameter mit Schieberegler mit je einer Gruppe fuer jeden Parameter."));
    settings->endGroup();


    for (int i = 0; i <= Enumerations::mue_ab; ++i){
        settings->beginGroup(dst->parsMeaning->at(i).symbol);
        for (int j = 0; j < Enumerations::countOf_parType; ++j)
            settings->setValue(dst->parsType->at(j), dst->pars->getParameterValue(Enumerations::parName(i), Enumerations::parType(j)));
        settings->endGroup();
    }

    settings->beginGroup(tr("Parameter-Ende"));
    settings->setValue(tr("HINWEIS"), tr("Nach diesem Hinweis folgen die Werte fuer die Optionen der Anpassungskurve in einer Gruppe gefolgt vom Namen der Vorlage."));
    settings->endGroup();

    //BEGIN DO NOT TRANSLATE

    settings->beginGroup("fit-options");
    settings->setValue("useA", dst->prgpar->useA);
    settings->setValue("useGradient", dst->prgpar->useGradient);
    settings->endGroup();

    settings->beginGroup("template-name");
    settings->setValue("name", *templateName);
    settings->endGroup();

    settings->beginGroup("X-Y");

    //END DO NOT TRANSLATE

    settings->setValue(tr("HINWEIS"), tr("Nach diesem Hinweis folgen die X-Y-Werte der Vorlage mit der zugehoerigen Wellenlaenge / Probendicke."));

    //BEGIN DO NOT TRANSLATE

    QVector<QPointF> *XYData = new QVector<QPointF>(0);

    datacal->calcFitCurve(XYData, dst->pars, 0.01, (floor(( (2 * sin(70 * M_PI / 180)) / (dst->prgpar->currentWavelength->value) ) * 10.) / 10.), 1., dst->prgpar->currentWavelength->value, dst->prgpar->useA, dst->prgpar->par_d, dst->prgpar->useGradient, false, dst->prgpar->par_r, dst->prgpar->par_delta, dst->prgpar->par_l, false, dst->prgpar->par_b);

    for (int i = 0; i < XYData->size(); ++i){
        settings->setValue("X_" + QString::number(XYData->at(i).x(), 'f', 3), XYData->at(i).x());
        settings->setValue("Y_" + QString::number(XYData->at(i).x(), 'f', 3), XYData->at(i).y());
    }

    settings->setValue("wavelength-value", dst->prgpar->currentWavelength->value);
    settings->setValue("sample-thickness", dst->prgpar->par_d);

     //END DO NOT TRANSLATE

    settings->endGroup();

    settings->sync();

    if ( !(settings->isWritable() && settings->status() == QSettings::NoError) )
        error = true;

    delete settings;
    delete templateName;
    delete writePath;

     return !error;
}


bool ParameterFileAccess::readAFS(AutoFitSequence *target, QString *readPath, DataStorage *dst){

    bool error = false; //no error initially

    //for temporary data
    QVariant tempVariant;

    QSettings *settings = new QSettings(*readPath, QSettings::IniFormat);

    settings->sync();

    //BEGIN DO NOT TRANSLATE

    settings->beginGroup("general-properties");

    tempVariant = settings->value("numberOfSteps");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Int))
        target->numberOfSteps = tempVariant.toInt();

    tempVariant = settings->value("sequenceName");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::String)){
        delete target->name;
        target->name = new QString(tempVariant.toString());
    }

    settings->endGroup();

    for (int i = 1; i <= target->numberOfSteps; ++i){

        AutoFitStep *newStep = new AutoFitStep();

        settings->beginGroup("step" + QString::number(i) + "-properties");

        tempVariant = settings->value("name");
        if (tempVariant.isValid() && tempVariant.canConvert(QVariant::String)){
            delete newStep->name;
            newStep->name = new QString(tempVariant.toString());
        }

        tempVariant = settings->value("useStep");
        if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool)){
            newStep->useStep = tempVariant.toBool();
        }

        settings->endGroup();

        for (int j = 0; j < Enumerations::countOf_parNameAutoFit; ++j){

            settings->beginGroup("step" + QString::number(i) + "-" + dst->parsMeaning->at( ( (j <= 12) ? (j) : (j+2) ) ).symbol);//cN and cO left out

            tempVariant = settings->value("autoFit");
            if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
                newStep->autoFitElements[j].autoFit = tempVariant.toBool();

            tempVariant = settings->value("min");
            if (tempVariant.isValid() && tempVariant.canConvert(QVariant::String)){
                QString *tempText = new QString(tempVariant.toString());
                if (AuxiliaryMethods::isDouble(tempText)){
                    newStep->autoFitElements[j].min = tempVariant.toDouble();
                    newStep->autoFitElements[j].useMin = true;
                }
                else{
                    newStep->autoFitElements[j].min = dst->pars->getParameterValue(Enumerations::parName( ( (j <= 12) ? (j) : (j+2) ) ), Enumerations::min);
                    newStep->autoFitElements[j].useMin = false;
                }
                delete tempText;
            }

            tempVariant = settings->value("max");
            if (tempVariant.isValid() && tempVariant.canConvert(QVariant::String)){
                QString *tempText = new QString(tempVariant.toString());
                if (AuxiliaryMethods::isDouble(tempText)){
                    newStep->autoFitElements[j].max = tempVariant.toDouble();
                    newStep->autoFitElements[j].useMax = true;
                }
                else{
                    newStep->autoFitElements[j].max = dst->pars->getParameterValue(Enumerations::parName( ( (j <= 12) ? (j) : (j+2) ) ), Enumerations::max);
                    newStep->autoFitElements[j].useMax = false;
                }
            }

            settings->endGroup();
        }

        settings->beginGroup("step" + QString::number(i) + "-additionalConditions");

        int numberOfAdditionalConditions = 0;
        tempVariant = settings->value("numberOfAdditionalConditions");
        if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Int))
            numberOfAdditionalConditions = tempVariant.toInt();

        for (int j = 1; j <= numberOfAdditionalConditions; ++j){
            tempVariant = settings->value("additionalCondition" + QString::number(j));
            if (tempVariant.isValid() && tempVariant.canConvert(QVariant::String)){
                newStep->additionalConditions->append(tempVariant.toString());
            }
        }

        settings->endGroup();

        target->autoFitSteps->append(*newStep);

        delete newStep;

    }

    //END DO NOT TRANSLATE

    if ( ! ( (settings->status() == QSettings::NoError) && (QFile::exists(*readPath) ) ) )
        error = true;

    delete settings;

    delete readPath;

    return !error;
}


QString* ParameterFileAccess::readAFSName(QString *readPath){

    QString *toReturn = new QString();

    //for temporary data
    QVariant tempVariant;

    QSettings *settings = new QSettings(*readPath, QSettings::IniFormat);

    settings->sync();

    //BEGIN DO NOT TRANSLATE

    settings->beginGroup("general-properties");

    tempVariant = settings->value("sequenceName");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::String)){
        delete toReturn;
        toReturn = new QString(tempVariant.toString());
    }

    //END DO NOT TRANSLATE

    settings->endGroup();

    delete settings;
    delete readPath;

    return toReturn;

}


bool ParameterFileAccess::writeAFS(AutoFitSequence *toWrite, QString *writePath, DataStorage *dst){
    bool error = false; //no error initially

    QSettings *settings = new QSettings(*writePath, QSettings::IniFormat);

    settings->clear();
    settings->sync();

    settings->beginGroup(tr("WARNUNG"));
    settings->setValue(tr("WARNUNG1"), tr("---Diese Datei darf nicht manuell veraendert werden!---"));
    settings->setValue(tr("WARNUNG2"), tr("---Manuelle Aenderungen koennen beim Laden der Datei Abstuerze und Datenverlust verursachen!---"));
    settings->endGroup();

    //BEGIN DO NOT TRANSLATE EXCEPT "KEIN"

    settings->beginGroup("general-properties");

    settings->setValue("numberOfSteps", toWrite->numberOfSteps);
    settings->setValue("sequenceName", *(toWrite->name));

    settings->endGroup();

    for (int i = 0; i < toWrite->numberOfSteps; ++i){

        settings->beginGroup("step" + QString::number(i+1) + "-properties");
        settings->setValue("name", *(toWrite->autoFitSteps->at(i).name));
        settings->setValue("useStep", toWrite->autoFitSteps->at(i).useStep);
        settings->endGroup();

        for (int j = 0; j < Enumerations::countOf_parNameAutoFit; ++j){

            settings->beginGroup("step" + QString::number(i+1) + "-" + dst->parsMeaning->at( ( (j <= 12) ? (j) : (j+2) ) ).symbol);//cN and cO left out

            settings->setValue("autoFit", toWrite->autoFitSteps->at(i).autoFitElements[j].autoFit);

            if (toWrite->autoFitSteps->at(i).autoFitElements[j].useMin)
                settings->setValue("min", toWrite->autoFitSteps->at(i).autoFitElements[j].min);
            else
                settings->setValue("min", tr("kein"));

            if (toWrite->autoFitSteps->at(i).autoFitElements[j].useMax)
                settings->setValue("max", toWrite->autoFitSteps->at(i).autoFitElements[j].max);
            else
                settings->setValue("max", tr("kein"));

            settings->endGroup();
        }

        settings->beginGroup("step" + QString::number(i+1) + "-additionalConditions");

        settings->setValue("numberOfAdditionalConditions", toWrite->autoFitSteps->at(i).additionalConditions->size());

        for (int j = 0; j < toWrite->autoFitSteps->at(i).additionalConditions->size(); ++j)
            settings->setValue("additionalCondition" + QString::number(j+1), toWrite->autoFitSteps->at(i).additionalConditions->at(j));

        settings->endGroup();

    }

    //END DO NOT TRANSLATE

    settings->sync();

    if ( !(settings->isWritable() && (settings->status() == QSettings::NoError) ) )
        error = true;

    delete settings;

    delete writePath;

    return !error;
}

