/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "parameterset.h"

ParameterSet::ParameterSet(QWidget *parentWidget, QObject *parent) : QObject(parent){
    this->parent = parentWidget;
    setFailSafeValues();
}

ParameterSet::~ParameterSet(){
}

double ParameterSet::getParameterValue(Enumerations::parName name, Enumerations::parType type){
    return parameters[name][type];
}

void ParameterSet::setParameterValue(Enumerations::parName name, Enumerations::parType type, double value, bool checkValue){
    if (checkValue){
        switch (name){
            case Enumerations::mu :
                switch (type){
                    case Enumerations::max :
                        if (value > 10 && value > parameters[Enumerations::mu][Enumerations::max]){
                            QMessageBox *msg = new QMessageBox(parent);
                            msg->setIcon(QMessageBox::Warning);
                            msg->setText(tr("Bei Werten von mu > 10 koennen Probleme wie\nPolstellen in der Anpassungskurve auftreten.\nEs ist daher sinnvoller beta anzupassen.\nMoechten Sie dennoch groessere Werte erlauben?\n[Ja]: groessere Werte zulassen\n[Nein]: auf empfohlenen Maximalwert (10) einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                            msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                            msg->setDefaultButton(QMessageBox::No);
                            int ret = msg->exec();
                            switch (ret) {
                               case QMessageBox::Yes:
                                   parameters[Enumerations::mu][Enumerations::max] = value;
                                   break;
                               case QMessageBox::No:
                                   parameters[Enumerations::mu][Enumerations::max] = 10;
                                   break;
                               case QMessageBox::Cancel:
                                   break;
                               default:
                                   parameters[Enumerations::mu][Enumerations::max] = 10;
                                   break;
                            }
                            delete msg;
                        }
                        else
                            parameters[Enumerations::mu][Enumerations::max] = value;
                        break;
                    default :
                        parameters[Enumerations::mu][type] = value;
                        break;
                }
                break;
            case Enumerations::a3 :
                switch (type){
                    case Enumerations::val :
                        if ( ( (value - parameters[Enumerations::da3][Enumerations::val]) < 2.7 ) && (value < parameters[Enumerations::a3][Enumerations::val])){
                            QMessageBox *msg = new QMessageBox(parent);
                            msg->setIcon(QMessageBox::Warning);
                            msg->setText(tr("Bei Werten von a3min (= a3-da3) < 2.7 koennen\nProbleme mit der Anpassungskurve auftreten.\nMoechten Sie dennoch a3 auf diesen Wert setzen?\n[Ja]: a3 auf diesen Wert setzen\n[Nein]: a3 auf empfohlenen Minimalwert einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                            msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                            msg->setDefaultButton(QMessageBox::No);
                            int ret = msg->exec();
                            switch (ret) {
                                case QMessageBox::Yes:
                                    parameters[Enumerations::a3][Enumerations::val] = value;
                                    break;
                                case QMessageBox::No:
                                    parameters[Enumerations::a3][Enumerations::val] = 2.7 + parameters[Enumerations::da3][Enumerations::val];
                                    break;
                                case QMessageBox::Cancel:
                                    break;
                                default:
                                    parameters[Enumerations::a3][Enumerations::val] = 2.7 + parameters[Enumerations::da3][Enumerations::val];
                                    break;
                            }
                            delete msg;
                        }
                        else
                            parameters[Enumerations::a3][Enumerations::val] = value;
                        break;
                    default :
                        parameters[Enumerations::a3][type] = value;
                        break;
                }
                break;
            case Enumerations::da3 :
                switch (type){
                    case Enumerations::val :
                        if ( ( (parameters[Enumerations::a3][Enumerations::val] - value) < 2.7 ) && ( (value / parameters[Enumerations::sig3][Enumerations::val]) <= 0.3) ){
                            QMessageBox *msg = new QMessageBox(parent);
                            msg->setIcon(QMessageBox::Warning);
                            msg->setText(tr("Bei Werten von a3min (= a3-da3) < 2.7 und da3/sig3 <= 0.3 koennen\nProbleme mit der Anpassungskurve auftreten.\nMoechten Sie dennoch da3 auf diesen Wert setzen?\n[Ja]: da3 auf diesen Wert setzen\n[Abbrechen]: vorherigen Wert beibehalten"));
                            msg->setStandardButtons(QMessageBox::Yes | QMessageBox::Cancel);
                            msg->setDefaultButton(QMessageBox::Cancel);
                            int ret = msg->exec();
                            switch (ret) {
                                case QMessageBox::Yes:
                                    parameters[Enumerations::da3][Enumerations::val] = value;
                                    break;
                                case QMessageBox::Cancel:
                                    break;
                                default:
                                    break;
                            }
                            delete msg;
                        }
                        else {
                            if ( ( (parameters[Enumerations::a3][Enumerations::val] - value) < 2.7 ) && (value > parameters[Enumerations::da3][Enumerations::val])){
                                double da3_recommended = parameters[Enumerations::a3][Enumerations::val] - 2.7;
                                if ( ( (da3_recommended / parameters[Enumerations::sig3][Enumerations::val]) <= 0.3) ){
                                    QMessageBox *msg = new QMessageBox(parent);
                                    msg->setIcon(QMessageBox::Warning);
                                    msg->setText(tr("Bei Werten von a3min (= a3-da3) < 2.7 koennen\nProbleme mit der Anpassungskurve auftreten.\nMoechten Sie dennoch da3 auf diesen Wert setzen?\n[Ja]: da3 auf diesen Wert setzen\n[Abbrechen]: vorherigen Wert beibehalten"));
                                    msg->setStandardButtons(QMessageBox::Yes | QMessageBox::Cancel);
                                    msg->setDefaultButton(QMessageBox::Cancel);
                                    int ret = msg->exec();
                                    switch (ret) {
                                        case QMessageBox::Yes:
                                            parameters[Enumerations::da3][Enumerations::val] = value;
                                            break;
                                        case QMessageBox::Cancel:
                                            break;
                                        default:
                                            break;
                                    }
                                    delete msg;
                                }
                                    else{
                                        QMessageBox *msg = new QMessageBox(parent);
                                        msg->setIcon(QMessageBox::Warning);
                                        msg->setText(tr("Bei Werten von a3min (= a3-da3) < 2.7 koennen\nProbleme mit der Anpassungskurve auftreten.\nMoechten Sie dennoch da3 auf diesen Wert setzen?\n[Ja]: da3 auf diesen Wert setzen\n[Nein]: da3 auf empfohlenen Maximalwert einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                                        msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                                        msg->setDefaultButton(QMessageBox::No);
                                        int ret = msg->exec();
                                        switch (ret) {
                                            case QMessageBox::Yes:
                                                parameters[Enumerations::da3][Enumerations::val] = value;
                                                break;
                                            case QMessageBox::No:
                                                parameters[Enumerations::da3][Enumerations::val] = da3_recommended;
                                                break;
                                            case QMessageBox::Cancel:
                                                break;
                                            default:
                                                parameters[Enumerations::da3][Enumerations::val] = da3_recommended;
                                                break;
                                        }
                                        delete msg;
                                    }
                            }
                            else {
                                if ( ( (value / parameters[Enumerations::sig3][Enumerations::val]) <= 0.3) && (value < parameters[Enumerations::da3][Enumerations::val])){
                                    double da3_recommended = 0.3 * parameters[Enumerations::sig3][Enumerations::val];
                                    if ( ( (parameters[Enumerations::a3][Enumerations::val] - da3_recommended) < 2.7 ) ){
                                        QMessageBox *msg = new QMessageBox(parent);
                                        msg->setIcon(QMessageBox::Warning);
                                        msg->setText(tr("Bei Werten von da3/sig3 <= 0.3 koennen\nProbleme mit der Anpassungskurve auftreten.\nMoechten Sie dennoch da3 auf diesen Wert setzen?\n[Ja]: da3 auf diesen Wert setzen\n[Abbrechen]: vorherigen Wert beibehalten"));
                                        msg->setStandardButtons(QMessageBox::Yes | QMessageBox::Cancel);
                                        msg->setDefaultButton(QMessageBox::Cancel);
                                        int ret = msg->exec();
                                        switch (ret) {
                                            case QMessageBox::Yes:
                                                parameters[Enumerations::da3][Enumerations::val] = value;
                                                break;
                                            case QMessageBox::Cancel:
                                                break;
                                            default:
                                                break;
                                        }
                                        delete msg;
                                    }
                                        else{
                                            QMessageBox *msg = new QMessageBox(parent);
                                            msg->setIcon(QMessageBox::Warning);
                                            msg->setText(tr("Bei Werten von da3/sig3 <= 0.3 koennen\nProbleme mit der Anpassungskurve auftreten.\nMoechten Sie dennoch da3 auf diesen Wert setzen?\n[Ja]: da3 auf diesen Wert setzen\n[Nein]: da3 auf empfohlenen Minimalwert einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                                            msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                                            msg->setDefaultButton(QMessageBox::No);
                                            int ret = msg->exec();
                                            switch (ret) {
                                                case QMessageBox::Yes:
                                                    parameters[Enumerations::da3][Enumerations::val] = value;
                                                    break;
                                                case QMessageBox::No:
                                                    parameters[Enumerations::da3][Enumerations::val] = da3_recommended;
                                                    break;
                                                case QMessageBox::Cancel:
                                                    break;
                                                default:
                                                    parameters[Enumerations::da3][Enumerations::val] = da3_recommended;
                                                    break;
                                            }
                                            delete msg;
                                        }
                                }
                                else
                                    parameters[Enumerations::da3][Enumerations::val] = value;
                            }
                        }
                        break;
                    case Enumerations::max :
                        if (value > 1 && value > parameters[Enumerations::da3][Enumerations::max]){
                            QMessageBox *msg = new QMessageBox(parent);
                            msg->setIcon(QMessageBox::Warning);
                            msg->setText(tr("Bei Werten von da3 > 1 koennen Probleme wie\nPolstellen in der Anpassungskurve auftreten.\nMoechten Sie dennoch groessere Werte erlauben?\n[Ja]: groessere Werte zulassen\n[Nein]: auf empfohlenen Maximalwert (1) einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                            msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                            msg->setDefaultButton(QMessageBox::No);
                            int ret = msg->exec();
                            switch (ret) {
                               case QMessageBox::Yes:
                                   parameters[Enumerations::da3][Enumerations::max] = value;
                                   break;
                               case QMessageBox::No:
                                   parameters[Enumerations::da3][Enumerations::max] = 1;
                                   break;
                               case QMessageBox::Cancel:
                                   break;
                               default:
                                   parameters[Enumerations::da3][Enumerations::max] = 1;
                                   break;
                            }
                            delete msg;
                        }
                        else
                            parameters[Enumerations::da3][Enumerations::max] = value;
                        break;
                    default :
                        parameters[Enumerations::da3][type] = value;
                        break;
                }
                break;
            case Enumerations::sig3 :
                switch (type){
                    case Enumerations::val :
                        if ( ( (parameters[Enumerations::da3][Enumerations::val] / value) <= 0.3 ) && (value > parameters[Enumerations::sig3][Enumerations::val])){
                            QMessageBox *msg = new QMessageBox(parent);
                            msg->setIcon(QMessageBox::Warning);
                            msg->setText(tr("Bei Werten von da3/sig3 <= 0.3 koennen\nProbleme mit der Anpassungskurve auftreten.\nMoechten Sie dennoch sig3 auf diesen Wert setzen?\n[Ja]: sig3 auf diesen Wert setzen\n[Nein]: sig3 auf empfohlenen Maximalwert einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                            msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                            msg->setDefaultButton(QMessageBox::No);
                            int ret = msg->exec();
                            switch (ret) {
                                case QMessageBox::Yes:
                                    parameters[Enumerations::sig3][Enumerations::val] = value;
                                    break;
                                case QMessageBox::No:
                                    parameters[Enumerations::sig3][Enumerations::val] =  parameters[Enumerations::da3][Enumerations::val] / 0.3;
                                    break;
                                case QMessageBox::Cancel:
                                    break;
                                default:
                                    parameters[Enumerations::sig3][Enumerations::val] =  parameters[Enumerations::da3][Enumerations::val] / 0.3;
                                    break;
                            }
                            delete msg;
                        }
                        else
                            parameters[Enumerations::sig3][Enumerations::val] = value;
                        break;
                    case Enumerations::max :
                        if (value > 2 && value > parameters[Enumerations::sig3][Enumerations::max]){
                            QMessageBox *msg = new QMessageBox(parent);
                            msg->setIcon(QMessageBox::Warning);
                            msg->setText(tr("Bei Werten von sig3 > 2 koennen Probleme wie\nPolstellen in der Anpassungskurve auftreten.\nMoechten Sie dennoch groessere Werte erlauben?\n[Ja]: groessere Werte zulassen\n[Nein]: auf empfohlenen Maximalwert (2) einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                            msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                            msg->setDefaultButton(QMessageBox::No);
                            int ret = msg->exec();
                            switch (ret) {
                               case QMessageBox::Yes:
                                   parameters[Enumerations::sig3][Enumerations::max] = value;
                                   break;
                               case QMessageBox::No:
                                   parameters[Enumerations::sig3][Enumerations::max] = 2;
                                   break;
                               case QMessageBox::Cancel:
                                   break;
                               default:
                                   parameters[Enumerations::sig3][Enumerations::max] = 2;
                                   break;
                            }
                            delete msg;
                        }
                        else
                            parameters[Enumerations::sig3][Enumerations::max] = value;
                        break;
                    default :
                        parameters[Enumerations::sig3][type] = value;
                        break;
                }
                break;
            case Enumerations::nu :
                switch (type){
                    case Enumerations::max :
                        if (value > 20 && value > parameters[Enumerations::nu][Enumerations::max]){
                            QMessageBox *msg = new QMessageBox(parent);
                            msg->setIcon(QMessageBox::Warning);
                            msg->setText(tr("Bei Werten von nu > 20 koennen Probleme wie\nPolstellen in der Anpassungskurve auftreten.\nEs ist daher sinnvoller alpha anzupassen.\nMoechten Sie dennoch groessere Werte erlauben?\n[Ja]: groessere Werte zulassen\n[Nein]: auf empfohlenen Maximalwert (20) einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                            msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                            msg->setDefaultButton(QMessageBox::No);
                            int ret = msg->exec();
                            switch (ret) {
                               case QMessageBox::Yes:
                                   parameters[Enumerations::nu][Enumerations::max] = value;
                                   break;
                               case QMessageBox::No:
                                   parameters[Enumerations::nu][Enumerations::max] = 20;
                                   break;
                               case QMessageBox::Cancel:
                                   break;
                               default:
                                   parameters[Enumerations::nu][Enumerations::max] = 20;
                                   break;
                            }
                            delete msg;
                        }
                        else
                            parameters[Enumerations::nu][Enumerations::max] = value;
                        break;
                    default:
                        parameters[Enumerations::nu][type] = value;
                        break;
                }
                break;
            default :
                parameters[name][type] = value;
                break;
        }
    }
    else
        parameters[name][type] = value;
}

void ParameterSet::setFailSafeValues(){
    for (int i = 0; i < Enumerations::countOf_parName; ++i){
        parameters[i][Enumerations::val] = 5;
        parameters[i][Enumerations::min] = 0;
        parameters[i][Enumerations::max] = 10;
        parameters[i][Enumerations::dec] = 1;
        parameters[i][Enumerations::inc] = 1;
    }
}

ParameterSet* ParameterSet::duplicate(){
    ParameterSet *duplicate = new ParameterSet(parent);
    for (int i = 0; i < Enumerations::countOf_parName; ++i)
        for (int j = 0; j < Enumerations::countOf_parType; ++j)
            duplicate->setParameterValue(Enumerations::parName(i), Enumerations::parType(j), parameters[i][j], false);
    return duplicate;
}

ParameterSetAutoFit* ParameterSet::duplicateForAutoFit(){
    ParameterSetAutoFit *duplicate = new ParameterSetAutoFit(parent);
    duplicate->setParameterValue(Enumerations::cnoAF, parameters[Enumerations::cno][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::muAF, parameters[Enumerations::mu][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::betaAF, parameters[Enumerations::beta][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::a3AF, parameters[Enumerations::a3][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::da3AF, parameters[Enumerations::da3][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::sig3AF, parameters[Enumerations::sig3][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::u3AF, parameters[Enumerations::u3][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::etaAF, parameters[Enumerations::eta][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::nuAF, parameters[Enumerations::nu][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::alphaAF, parameters[Enumerations::alpha][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::sig1AF, parameters[Enumerations::sig1][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::lccAF, parameters[Enumerations::lcc][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::qAF, parameters[Enumerations::q][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::danAF, parameters[Enumerations::dan][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::kAF, parameters[Enumerations::k][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::const1AF, parameters[Enumerations::const1][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::const2AF, parameters[Enumerations::const2][Enumerations::val]);
    duplicate->setParameterValue(Enumerations::gAF, parameters[Enumerations::g][Enumerations::val]);
    return duplicate;
}
