/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "mainwindow.h"

#include "autofitsettings.h"
#include "ui_autofitsettings.h"

AutoFitSettings::AutoFitSettings(DataStorage *dst, SettingsStorage *sst, ParameterFileAccess *pfa, MainWindow *mw, QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::AutoFitSettings){

    ui->setupUi(this);

    this->dst = dst;
    this->sst = sst;
    this->pfa = pfa;
    this->mw  = mw;

    dst->parsAutoFit = dst->pars->duplicateForAutoFit();

    brush_activeStep = new QBrush(QColor(0,255,0,80));
    brush_inactiveStep = new QBrush(QColor(255,0,0,80));

    must_takeOverResults = false;
    must_saveSequence = false;

    //update comboBox AutoFitSequences
    bool prevState = ui->comboBox_AutoFitSequences->blockSignals(true); //save previous state
    QDir *dir = new QDir(*(dst->applicationDirPath) + "/data/autofitsequences/");
    dir->setFilter(QDir::Files);
    QStringList *nameFilters = new QStringList("*.afs");
    dir->setNameFilters(*nameFilters);
    QFileInfoList list =  dir->entryInfoList();
    for (int i = 0; i < list.size(); ++i)
        if (list.at(i).exists()){
            QString *toAdd = pfa->readAFSName(new QString(list.at(i).absoluteFilePath()));
            if (!(toAdd->isEmpty()))
                ui->comboBox_AutoFitSequences->addItem(*toAdd, list.at(i).absoluteFilePath());
            delete toAdd;
        }
    delete nameFilters;
    delete dir;
    ui->comboBox_AutoFitSequences->blockSignals(prevState); //restore previous state

    afseq = new AutoFitSequence();
    if (ui->comboBox_AutoFitSequences->currentIndex() >= 0){
        int posOfDefault = ui->comboBox_AutoFitSequences->findText("Standard / default");
        if (posOfDefault >= 1)
            ui->comboBox_AutoFitSequences->setCurrentIndex(posOfDefault);
        else //element at first position or not found
            on_comboBox_AutoFitSequences_currentIndexChanged(0);
    }
    else
        on_comboBox_AutoFitSequences_currentIndexChanged(-1);

    if ( !(dst->prgpar->useGradient) ){
        ui->widget_g->hide();
        ui->widget_k->move(170, 770);
        ui->widget_const1->move(290, 580);
        ui->widget_const2->move(290, 675);
    }

    setParameterWidgetValues(); //values of the parameters

    setFixedSize(width(), height());
    setWindowFlags( (windowFlags() | Qt::CustomizeWindowHint) & ~Qt::WindowMaximizeButtonHint);

}


AutoFitSettings::~AutoFitSettings(){
    delete ui;

    delete afseq;

    delete brush_inactiveStep;
    delete brush_activeStep;

    delete dst->parsAutoFit;
    dst->parsAutoFit = 0;
}


void AutoFitSettings::setDefaultAutoFitSequence(){
    delete afseq;
    afseq = new AutoFitSequence;
    afseq->numberOfSteps = 1;
    delete afseq->name;
    afseq->name = new QString ("Standard / default");
    AutoFitStep *newStep = new AutoFitStep();
    delete newStep->name;
    newStep->name = new QString("alles / complete");
    for (int i = 0; i < Enumerations::countOf_parNameAutoFit; ++i){
        newStep->autoFitElements[i].autoFit = true;
        newStep->autoFitElements[i].min = dst->pars->getParameterValue(Enumerations::parName( ( (i <= 12) ? (i) : (i+2) ) ), Enumerations::min);
        newStep->autoFitElements[i].max = dst->pars->getParameterValue(Enumerations::parName( ( (i <= 12) ? (i) : (i+2) ) ), Enumerations::max);
        newStep->autoFitElements[i].useMin = true;
        newStep->autoFitElements[i].useMax = true;
    }
    afseq->autoFitSteps->append(*newStep);
    delete newStep;
}


void AutoFitSettings::setParameterWidgetValues(){
    ui->cnoValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::cnoAF),'f',5));
    ui->muValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::muAF),'f',5));
    ui->betaValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::betaAF),'f',5));
    ui->a3Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::a3AF),'f',5));
    ui->da3Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::da3AF),'f',5));
    ui->sig3Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::sig3AF),'f',5));
    ui->u3Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::u3AF),'f',5));
    ui->etaValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::etaAF),'f',5));
    ui->nuValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::nuAF),'f',0));
    ui->alphaValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::alphaAF),'f',5));
    ui->sig1Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::sig1AF),'f',5));
    ui->lccValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::lccAF),'f',5));
    ui->qValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::qAF),'f',5));
    ui->danValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::danAF),'f',5));
    ui->kValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::kAF),'f',5));
    ui->const1Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::const1AF),'f',5));
    ui->const2Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::const2AF),'f',5));
    ui->gValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::gAF),'f',5));

    if ( ui->cno->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMax ) ) )
        ui->cnoValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->cnoValue->setStyleSheet("");

    if ( ui->mu->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::muAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::muAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMax ) ) )
        ui->muValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->muValue->setStyleSheet("");

    if ( ui->beta->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMax ) ) )
        ui->betaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->betaValue->setStyleSheet("");

    if ( ui->a3->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMax ) ) )
        ui->a3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->a3Value->setStyleSheet("");

    if ( ui->da3->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMax ) ) )
        ui->da3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->da3Value->setStyleSheet("");

    if ( ui->sig3->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMax ) ) )
        ui->sig3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->sig3Value->setStyleSheet("");

    if ( ui->u3->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMax ) ) )
        ui->u3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->u3Value->setStyleSheet("");

    if ( ui->eta->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMax ) ) )
        ui->etaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->etaValue->setStyleSheet("");

    if ( ui->nu->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMax ) ) )
        ui->nuValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->nuValue->setStyleSheet("");

    if ( ui->alpha->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMax ) ) )
        ui->alphaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->alphaValue->setStyleSheet("");

    if ( ui->sig1->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMax ) ) )
        ui->sig1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->sig1Value->setStyleSheet("");

    if ( ui->lcc->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMax ) ) )
        ui->lccValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->lccValue->setStyleSheet("");

    if ( ui->q->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::qAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::qAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMax ) ) )
        ui->qValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->qValue->setStyleSheet("");

    if ( ui->dan->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::danAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::danAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMax ) ) )
        ui->danValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->danValue->setStyleSheet("");

    if ( ui->k->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::kAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::kAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMax ) ) )
        ui->kValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->kValue->setStyleSheet("");

    if ( ui->const1->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMax ) ) )
        ui->const1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->const1Value->setStyleSheet("");

    if ( ui->const2->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMax ) ) )
        ui->const2Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->const2Value->setStyleSheet("");

    if ( ui->g->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::gAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::gAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMax ) ) )
        ui->gValue->setStyleSheet("backconst2round-color: rconst2b(255, 255, 0);");
    else
        ui->gValue->setStyleSheet("");
}


bool AutoFitSettings::existsSavePath(QString *name){
    bool exists = false;
    QString *savePathFromName = new QString(*name);
    savePathFromName->replace(" ", "_", Qt::CaseInsensitive);
    savePathFromName->replace("/", "_", Qt::CaseInsensitive);
    savePathFromName->replace("\\", "_", Qt::CaseInsensitive);
    savePathFromName->replace(":", "_", Qt::CaseInsensitive);
    savePathFromName->replace("*", "_", Qt::CaseInsensitive);
    savePathFromName->replace("?", "_", Qt::CaseInsensitive);
    savePathFromName->replace("\"", "_", Qt::CaseInsensitive);
    savePathFromName->replace("<", "_", Qt::CaseInsensitive);
    savePathFromName->replace(">", "_", Qt::CaseInsensitive);
    savePathFromName->replace("|", "_", Qt::CaseInsensitive);
    savePathFromName->replace(QRegExp("_{2,}"), "_");
    savePathFromName->insert(0, *(dst->applicationDirPath) + "/data/autofitsequences/");
    savePathFromName->append(".afs");
    for (int i = 0; i < ui->comboBox_AutoFitSequences->count(); ++i){
        if ((savePathFromName->compare(ui->comboBox_AutoFitSequences->itemData(i).toString(), Qt::CaseInsensitive)) == 0)
            exists = true;
        QString *savePathFromComboBox = new QString(ui->comboBox_AutoFitSequences->itemText(i));
        savePathFromComboBox->replace(" ", "_", Qt::CaseInsensitive);
        savePathFromComboBox->replace("/", "_", Qt::CaseInsensitive);
        savePathFromComboBox->replace("\\", "_", Qt::CaseInsensitive);
        savePathFromComboBox->replace(":", "_", Qt::CaseInsensitive);
        savePathFromComboBox->replace("*", "_", Qt::CaseInsensitive);
        savePathFromComboBox->replace("?", "_", Qt::CaseInsensitive);
        savePathFromComboBox->replace("\"", "_", Qt::CaseInsensitive);
        savePathFromComboBox->replace("<", "_", Qt::CaseInsensitive);
        savePathFromComboBox->replace(">", "_", Qt::CaseInsensitive);
        savePathFromComboBox->replace("|", "_", Qt::CaseInsensitive);
        savePathFromComboBox->replace(QRegExp("_{2,}"), "_");
        savePathFromComboBox->insert(0, *(dst->applicationDirPath) + "/data/autofitsequences/");
        savePathFromComboBox->append(".afs");
        if ((savePathFromName->compare(savePathFromComboBox, Qt::CaseInsensitive)) == 0)
            exists = true;
        delete savePathFromComboBox;

    }
    delete savePathFromName;
    return exists;
}

void AutoFitSettings::saveAutoFitSequenceMaybe(){
    if (must_saveSequence &&  (ui->comboBox_AutoFitSequences->currentIndex() >= 0) ){
        QMessageBox *msg = new QMessageBox(this);
        msg->setIcon(QMessageBox::Question);
        msg->setText(tr("Moechten Sie die aktuelle Autofit-Sequenz speichern?"));
        msg->setStandardButtons(QMessageBox::Save | QMessageBox::Discard);
        msg->setDefaultButton(QMessageBox::Save);
        int ret = msg->exec();
        switch (ret) {
           case QMessageBox::Save:
               ui->buttonSaveAutoFitSequence->click();
               break;
           case QMessageBox::Discard:
               break;
           default:
               break;
         }
       delete msg;
    }
}


void AutoFitSettings::keyPressEvent(QKeyEvent *event){
    event->accept();
}


void AutoFitSettings::closeEvent(QCloseEvent *event){
    saveAutoFitSequenceMaybe();
    if (ui->buttonDisplayResults->isChecked())
        on_buttonDisplayResults_clicked(false);
    if (must_takeOverResults){
        QMessageBox *msg = new QMessageBox(this);
        msg->setIcon(QMessageBox::Question);
        msg->setText(tr("Sollen die Ergebnisse der automatischen Anpassung uebernommen werden?"));
        msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
        msg->setDefaultButton(QMessageBox::Yes);
        int ret = msg->exec();
        switch (ret) {
           case QMessageBox::Yes:
               ui->buttonTakeOverResults->click();
               break;
           case QMessageBox::No:
               break;
           default:
               break;
         }
       delete msg;
    }
    event->accept();
    mw->on_close_afs();
}



void AutoFitSettings::on_scrollBarControls_valueChanged(int value){
    int movement = -95 * value;

    //move by x = 0, y = movement + y(Widget)
    ui->widget_cno->move(10, movement + 10);
    ui->widget_mu->move(10, movement + 105);
    ui->widget_beta->move(10, movement + 200);
    ui->widget_a3->move(10, movement + 295);
    ui->widget_da3->move(10, movement + 390);
    ui->widget_sig3->move(10, movement + 485);
    ui->widget_u3->move(10, movement + 580);
    ui->widget_eta->move(10, movement + 675);


    //move by x = 380, y = movement + y(Widget)
    ui->widget_nu->move(290, movement + 10);
    ui->widget_alpha->move(290, movement + 105);
    ui->widget_sig1->move(290, movement + 200);
    ui->widget_lcc->move(290, movement + 295);
    ui->widget_q->move(290, movement + 390);
    ui->widget_dan->move(290, movement + 485);
    ui->widget_g->move(290, movement + 675);

    //move widget_const1 by either x = 0 or x = 380 (depending on availability of widget_g), y = movement + y(Widget)
    //move widget_const2 by x = 0, y = movement + y(Widget)
    //move widget_k by by either x = 290 or x = 170 (depending on availability of widget_g), y = movement + y(Widget)
    // y(Widget) also depends on availability of widget_g
    if (dst->prgpar->useGradient){
        ui->widget_const1->move(10, movement + 770);
        ui->widget_const2->move(290, movement + 770);
        ui->widget_k->move(290, movement + 580);
    }
    else{
        ui->widget_const1->move(290, movement + 580);
        ui->widget_const2->move(290, movement + 675);
        ui->widget_k->move(170, movement + 770);
    }
}


void AutoFitSettings::on_cno_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->cnoValue->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::cnoAF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->cnoMin->show();
        ui->label_cnoMin->show();
        ui->cnoMax->show();
        ui->label_cnoMax->show();
        ui->label_cnoValue->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMax ) )
            ui->cnoValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_cnoValue->setText(tr("Wert"));
        ui->cnoMin->hide();
        ui->label_cnoMin->hide();
        ui->cnoMax->hide();
        ui->label_cnoMax->hide();
    }
    must_saveSequence = true;
    ui->cno->setChecked(checked);

}

void AutoFitSettings::on_button_cnoValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::cnoAF, dst->pars->getParameterValue(Enumerations::cno, Enumerations::val));
    ui->cnoValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::cnoAF),'f',5));
    if ( ui->cno->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMax ) ) )
        ui->cnoValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->cnoValue->setStyleSheet("");
}

void AutoFitSettings::on_cnoMin_editingFinished(){
    QString *tempText = new QString(ui->cnoMin->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::cnoAF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::cnoAF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::cnoAF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::cnoAF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::cnoAF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->cnoValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMin){
        ui->cnoMin->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].max) )
            ui->cnoValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->cnoMin->setText(tr("kein"));
}

void AutoFitSettings::on_cnoMax_editingFinished(){
    QString *tempText = new QString(ui->cnoMax->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::cnoAF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::cnoAF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::cnoAF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::cnoAF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::cnoAF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->cnoValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].useMax){
        ui->cnoMax->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::cnoAF].min))
            ui->cnoValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->cnoMax->setText(tr("kein"));
}


void AutoFitSettings::on_mu_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->muValue->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->muMin->show();
        ui->label_muMin->show();
        ui->muMax->show();
        ui->label_muMax->show();
        ui->label_muValue->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::muAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::muAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMax ) )
            ui->muValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_muValue->setText(tr("Wert"));
        ui->muMin->hide();
        ui->label_muMin->hide();
        ui->muMax->hide();
        ui->label_muMax->hide();
    }
    must_saveSequence = true;
    ui->mu->setChecked(checked);
}

void AutoFitSettings::on_button_muValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::muAF, dst->pars->getParameterValue(Enumerations::mu, Enumerations::val));
    ui->muValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::muAF),'f',5));
    if ( ui->mu->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::muAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::muAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMax ) ) )
        ui->muValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->muValue->setStyleSheet("");
}

void AutoFitSettings::on_muMin_editingFinished(){
    QString *tempText = new QString(ui->muMin->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->muValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMin){
        ui->muMin->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::muAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::muAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].max) )
            ui->muValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->muMin->setText(tr("kein"));
}

void AutoFitSettings::on_muMax_editingFinished(){
    bool prevState = ui->muMax->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->muMax->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].min)
            if (temp > 10 && temp > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].max){
                QMessageBox *msg = new QMessageBox(this);
                msg->setIcon(QMessageBox::Warning);
                msg->setText(tr("Bei Werten von mu > 10 koennen Probleme wie\nPolstellen in der Anpassungskurve auftreten.\nEs ist daher sinnvoller beta anzupassen.\nMoechten Sie dennoch groessere Werte erlauben?\n[Ja]: groessere Werte zulassen\n[Nein]: auf empfohlenen Maximalwert (10) einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                msg->setDefaultButton(QMessageBox::No);
                int ret = msg->exec();
                switch (ret) {
                   case QMessageBox::Yes:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].max = temp;
                       break;
                   case QMessageBox::No:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].max = 10;
                       break;
                   case QMessageBox::Cancel:
                       break;
                   default:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].max = 10;
                       break;
                }
                delete msg;
            }
            else
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::muAF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->muValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].useMax){
        ui->muMax->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::muAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::muAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::muAF].min))
            ui->muValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->muMax->setText(tr("kein"));
    ui->muMax->blockSignals(prevState); //restore previous state
}


void AutoFitSettings::on_beta_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->betaValue->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::betaAF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->betaMin->show();
        ui->label_betaMin->show();
        ui->betaMax->show();
        ui->label_betaMax->show();
        ui->label_betaValue->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMax ) )
            ui->betaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_betaValue->setText(tr("Wert"));
        ui->betaMin->hide();
        ui->label_betaMin->hide();
        ui->betaMax->hide();
        ui->label_betaMax->hide();
    }
    must_saveSequence = true;
    ui->beta->setChecked(checked);
}

void AutoFitSettings::on_button_betaValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::betaAF, dst->pars->getParameterValue(Enumerations::beta, Enumerations::val));
    ui->betaValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::betaAF),'f',5));
    if ( ui->beta->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMax ) ) )
        ui->betaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->betaValue->setStyleSheet("");
}

void AutoFitSettings::on_betaMin_editingFinished(){
    QString *tempText = new QString(ui->betaMin->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::betaAF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::betaAF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::betaAF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::betaAF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::betaAF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->betaValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMin){
        ui->betaMin->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].max) )
            ui->betaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->betaMin->setText(tr("kein"));
}

void AutoFitSettings::on_betaMax_editingFinished(){
    QString *tempText = new QString(ui->betaMax->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::betaAF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::betaAF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::betaAF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::betaAF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::betaAF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->betaValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].useMax){
        ui->betaMax->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::betaAF].min))
            ui->betaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->betaMax->setText(tr("kein"));
}


void AutoFitSettings::on_a3_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->a3Value->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::a3AF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->a3Min->show();
        ui->label_a3Min->show();
        ui->a3Max->show();
        ui->label_a3Max->show();
        ui->label_a3Value->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMax ) )
            ui->a3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_a3Value->setText(tr("Wert"));
        ui->a3Min->hide();
        ui->label_a3Min->hide();
        ui->a3Max->hide();
        ui->label_a3Max->hide();
    }
    must_saveSequence = true;
    ui->a3->setChecked(checked);
}

void AutoFitSettings::on_button_a3ValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::a3AF, dst->pars->getParameterValue(Enumerations::a3, Enumerations::val));
    ui->a3Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::a3AF),'f',5));
    if ( ui->a3->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMax ) ) )
        ui->a3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->a3Value->setStyleSheet("");
}

void AutoFitSettings::on_a3Min_editingFinished(){
    QString *tempText = new QString(ui->a3Min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::a3AF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::a3AF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::a3AF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::a3AF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::a3AF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->a3Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMin){
        ui->a3Min->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].max) )
            ui->a3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->a3Min->setText(tr("kein"));
}

void AutoFitSettings::on_a3Max_editingFinished(){
    QString *tempText = new QString(ui->a3Max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::a3AF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::a3AF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::a3AF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::a3AF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::a3AF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->a3Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].useMax){
        ui->a3Max->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::a3AF].min))
            ui->a3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->a3Max->setText(tr("kein"));
}


void AutoFitSettings::on_da3_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->da3Value->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->da3Min->show();
        ui->label_da3Min->show();
        ui->da3Max->show();
        ui->label_da3Max->show();
        ui->label_da3Value->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMax ) )
            ui->da3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_da3Value->setText(tr("Wert"));
        ui->da3Min->hide();
        ui->label_da3Min->hide();
        ui->da3Max->hide();
        ui->label_da3Max->hide();
    }
    must_saveSequence = true;
    ui->da3->setChecked(checked);
}

void AutoFitSettings::on_button_da3ValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::da3AF, dst->pars->getParameterValue(Enumerations::da3, Enumerations::val));
    ui->da3Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::da3AF),'f',5));
    if ( ui->da3->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMax ) ) )
        ui->da3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->da3Value->setStyleSheet("");
}

void AutoFitSettings::on_da3Min_editingFinished(){
    QString *tempText = new QString(ui->da3Min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->da3Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMin){
        ui->da3Min->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].max) )
            ui->da3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->da3Min->setText(tr("kein"));
}

void AutoFitSettings::on_da3Max_editingFinished(){
    bool prevState = ui->da3Max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->da3Max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].min)
            if (temp > 1 && temp > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].max){
                QMessageBox *msg = new QMessageBox(this);
                msg->setIcon(QMessageBox::Warning);
                msg->setText(tr("Bei Werten von da3 > 1 koennen Probleme wie\nPolstellen in der Anpassungskurve auftreten.\nMoechten Sie dennoch groessere Werte erlauben?\n[Ja]: groessere Werte zulassen\n[Nein]: auf empfohlenen Maximalwert (1) einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                msg->setDefaultButton(QMessageBox::No);
                int ret = msg->exec();
                switch (ret) {
                   case QMessageBox::Yes:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].max = temp;
                       break;
                   case QMessageBox::No:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].max = 1;
                       break;
                   case QMessageBox::Cancel:
                       break;
                   default:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].max = 1;
                       break;
                }
                delete msg;
            }
            else
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::da3AF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->da3Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].useMax){
        ui->da3Max->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::da3AF].min))
            ui->da3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->da3Max->setText(tr("kein"));
    ui->da3Max->blockSignals(prevState); //restore previous state
}


void AutoFitSettings::on_sig3_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->sig3Value->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->sig3Min->show();
        ui->label_sig3Min->show();
        ui->sig3Max->show();
        ui->label_sig3Max->show();
        ui->label_sig3Value->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMax ) )
            ui->sig3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_sig3Value->setText(tr("Wert"));
        ui->sig3Min->hide();
        ui->label_sig3Min->hide();
        ui->sig3Max->hide();
        ui->label_sig3Max->hide();
    }
    must_saveSequence = true;
    ui->sig3->setChecked(checked);
}

void AutoFitSettings::on_button_sig3ValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::sig3AF, dst->pars->getParameterValue(Enumerations::sig3, Enumerations::val));
    ui->sig3Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::sig3AF),'f',5));
    if ( ui->sig3->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMax ) ) )
        ui->sig3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->sig3Value->setStyleSheet("");
}

void AutoFitSettings::on_sig3Min_editingFinished(){
    QString *tempText = new QString(ui->sig3Min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->sig3Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMin){
        ui->sig3Min->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].max) )
            ui->sig3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->sig3Min->setText(tr("kein"));
}

void AutoFitSettings::on_sig3Max_editingFinished(){
    bool prevState = ui->sig3Max->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->sig3Max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].min)
            if (temp > 2 && temp > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].max){
                QMessageBox *msg = new QMessageBox(this);
                msg->setIcon(QMessageBox::Warning);
                msg->setText(tr("Bei Werten von sig3 > 2 koennen Probleme wie\nPolstellen in der Anpassungskurve auftreten.\nMoechten Sie dennoch groessere Werte erlauben?\n[Ja]: groessere Werte zulassen\n[Nein]: auf empfohlenen Maximalwert (2) einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                msg->setDefaultButton(QMessageBox::No);
                int ret = msg->exec();
                switch (ret) {
                   case QMessageBox::Yes:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].max = temp;
                       break;
                   case QMessageBox::No:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].max = 2;
                       break;
                   case QMessageBox::Cancel:
                       break;
                   default:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].max = 2;
                       break;
                }
                delete msg;
            }
            else
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig3AF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->sig3Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].useMax){
        ui->sig3Max->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig3AF].min))
            ui->sig3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->sig3Max->setText(tr("kein"));
    ui->sig3Max->blockSignals(prevState); //restore previous state
}


void AutoFitSettings::on_u3_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->u3Value->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::u3AF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->u3Min->show();
        ui->label_u3Min->show();
        ui->u3Max->show();
        ui->label_u3Max->show();
        ui->label_u3Value->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMax ) )
            ui->u3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_u3Value->setText(tr("Wert"));
        ui->u3Min->hide();
        ui->label_u3Min->hide();
        ui->u3Max->hide();
        ui->label_u3Max->hide();
    }
    must_saveSequence = true;
    ui->u3->setChecked(checked);
}

void AutoFitSettings::on_button_u3ValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::u3AF, dst->pars->getParameterValue(Enumerations::u3, Enumerations::val));
    ui->u3Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::u3AF),'f',5));
    if ( ui->u3->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMax ) ) )
        ui->u3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->u3Value->setStyleSheet("");
}

void AutoFitSettings::on_u3Min_editingFinished(){
    QString *tempText = new QString(ui->u3Min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::u3AF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::u3AF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::u3AF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::u3AF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::u3AF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->u3Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMin){
        ui->u3Min->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].max) )
            ui->u3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->u3Min->setText(tr("kein"));
}

void AutoFitSettings::on_u3Max_editingFinished(){
    QString *tempText = new QString(ui->u3Max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::u3AF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::u3AF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::u3AF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::u3AF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::u3AF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->u3Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].useMax){
        ui->u3Max->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::u3AF].min))
            ui->u3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->u3Max->setText(tr("kein"));
}


void AutoFitSettings::on_eta_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->etaValue->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::etaAF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->etaMin->show();
        ui->label_etaMin->show();
        ui->etaMax->show();
        ui->label_etaMax->show();
        ui->label_etaValue->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMax ) )
            ui->etaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_etaValue->setText(tr("Wert"));
        ui->etaMin->hide();
        ui->label_etaMin->hide();
        ui->etaMax->hide();
        ui->label_etaMax->hide();
    }
    must_saveSequence = true;
    ui->eta->setChecked(checked);
}

void AutoFitSettings::on_button_etaValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::etaAF, dst->pars->getParameterValue(Enumerations::eta, Enumerations::val));
    ui->etaValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::etaAF),'f',5));
    if ( ui->eta->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMax ) ) )
        ui->etaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->etaValue->setStyleSheet("");
}

void AutoFitSettings::on_etaMin_editingFinished(){
    QString *tempText = new QString(ui->etaMin->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::etaAF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::etaAF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::etaAF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::etaAF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::etaAF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->etaValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMin){
        ui->etaMin->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].max) )
            ui->etaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->etaMin->setText(tr("kein"));
}

void AutoFitSettings::on_etaMax_editingFinished(){
    QString *tempText = new QString(ui->etaMax->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::etaAF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::etaAF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::etaAF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::etaAF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::etaAF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->etaValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].useMax){
        ui->etaMax->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::etaAF].min))
            ui->etaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->etaMax->setText(tr("kein"));
}


void AutoFitSettings::on_nu_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->nuValue->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->nuMin->show();
        ui->label_nuMin->show();
        ui->nuMax->show();
        ui->label_nuMax->show();
        ui->label_nuValue->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMax ) )
            ui->nuValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_nuValue->setText(tr("Wert"));
        ui->nuMin->hide();
        ui->label_nuMin->hide();
        ui->nuMax->hide();
        ui->label_nuMax->hide();
    }
    must_saveSequence = true;
    ui->nu->setChecked(checked);
}

void AutoFitSettings::on_button_nuValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::nuAF, dst->pars->getParameterValue(Enumerations::nu, Enumerations::val));
    ui->nuValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::nuAF),'f',0));
    if ( ui->nu->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMax ) ) )
        ui->nuValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->nuValue->setStyleSheet("");
}

void AutoFitSettings::on_nuMin_editingFinished(){
    QString *tempText = new QString(ui->nuMin->text());
    if (AuxiliaryMethods::isInt(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 0);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->nuValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMin){
        ui->nuMin->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].min,'f',0));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].max) )
            ui->nuValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->nuMin->setText(tr("kein"));
}

void AutoFitSettings::on_nuMax_editingFinished(){
    bool prevState = ui->nuMax->blockSignals(true); //save previous state
    QString *tempText = new QString(ui->nuMax->text());
    if (AuxiliaryMethods::isInt(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 0);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].min)
            if (temp > 20 && temp > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].max){
                QMessageBox *msg = new QMessageBox(this);
                msg->setIcon(QMessageBox::Warning);
                msg->setText(tr("Bei Werten von nu > 20 koennen Probleme wie\nPolstellen in der Anpassungskurve auftreten.\nEs ist daher sinnvoller alpha anzupassen.\nMoechten Sie dennoch groessere Werte erlauben?\n[Ja]: groessere Werte zulassen\n[Nein]: auf empfohlenen Maximalwert (20) einstellen\n[Abbrechen]: vorherigen Wert beibehalten"));
                msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
                msg->setDefaultButton(QMessageBox::No);
                int ret = msg->exec();
                switch (ret) {
                   case QMessageBox::Yes:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].max = temp;
                       break;
                   case QMessageBox::No:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].max = 20;
                       break;
                   case QMessageBox::Cancel:
                       break;
                   default:
                       (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].max = 20;
                       break;
                }
                delete msg;
            }
            else
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::nuAF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->nuValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].useMax){
        ui->nuMax->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].max,'f',0));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::nuAF].min))
            ui->nuValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->nuMax->setText(tr("kein"));
    ui->nuMax->blockSignals(prevState); //restore previous state
}


void AutoFitSettings::on_alpha_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->alphaValue->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::alphaAF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->alphaMin->show();
        ui->label_alphaMin->show();
        ui->alphaMax->show();
        ui->label_alphaMax->show();
        ui->label_alphaValue->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMax ) )
            ui->alphaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_alphaValue->setText(tr("Wert"));
        ui->alphaMin->hide();
        ui->label_alphaMin->hide();
        ui->alphaMax->hide();
        ui->label_alphaMax->hide();
    }
    must_saveSequence = true;
    ui->alpha->setChecked(checked);
}

void AutoFitSettings::on_button_alphaValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::alphaAF, dst->pars->getParameterValue(Enumerations::alpha, Enumerations::val));
    ui->alphaValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::alphaAF),'f',5));
    if ( ui->alpha->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMax ) ) )
        ui->alphaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->alphaValue->setStyleSheet("");
}

void AutoFitSettings::on_alphaMin_editingFinished(){
    QString *tempText = new QString(ui->alphaMin->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::alphaAF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::alphaAF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::alphaAF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::alphaAF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::alphaAF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->alphaValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMin){
        ui->alphaMin->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].max) )
            ui->alphaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->alphaMin->setText(tr("kein"));
}

void AutoFitSettings::on_alphaMax_editingFinished(){
    QString *tempText = new QString(ui->alphaMax->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::alphaAF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::alphaAF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::alphaAF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::alphaAF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::alphaAF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->alphaValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].useMax){
        ui->alphaMax->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::alphaAF].min))
            ui->alphaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->alphaMax->setText(tr("kein"));
}


void AutoFitSettings::on_sig1_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->sig1Value->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig1AF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->sig1Min->show();
        ui->label_sig1Min->show();
        ui->sig1Max->show();
        ui->label_sig1Max->show();
        ui->label_sig1Value->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMax ) )
            ui->sig1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_sig1Value->setText(tr("Wert"));
        ui->sig1Min->hide();
        ui->label_sig1Min->hide();
        ui->sig1Max->hide();
        ui->label_sig1Max->hide();
    }
    must_saveSequence = true;
    ui->sig1->setChecked(checked);
}

void AutoFitSettings::on_button_sig1ValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::sig1AF, dst->pars->getParameterValue(Enumerations::sig1, Enumerations::val));
    ui->sig1Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::sig1AF),'f',5));
    if ( ui->sig1->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMax ) ) )
        ui->sig1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->sig1Value->setStyleSheet("");
}

void AutoFitSettings::on_sig1Min_editingFinished(){
    QString *tempText = new QString(ui->sig1Min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig1AF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig1AF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig1AF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig1AF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig1AF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->sig1Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMin){
        ui->sig1Min->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].max) )
            ui->sig1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->sig1Min->setText(tr("kein"));
}

void AutoFitSettings::on_sig1Max_editingFinished(){
    QString *tempText = new QString(ui->sig1Max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig1AF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig1AF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig1AF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig1AF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::sig1AF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->sig1Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].useMax){
        ui->sig1Max->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::sig1AF].min))
            ui->sig1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->sig1Max->setText(tr("kein"));
}


void AutoFitSettings::on_lcc_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->lccValue->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::lccAF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->lccMin->show();
        ui->label_lccMin->show();
        ui->lccMax->show();
        ui->label_lccMax->show();
        ui->label_lccValue->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMax ) )
            ui->lccValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_lccValue->setText(tr("Wert"));
        ui->lccMin->hide();
        ui->label_lccMin->hide();
        ui->lccMax->hide();
        ui->label_lccMax->hide();
    }
    must_saveSequence = true;
    ui->lcc->setChecked(checked);
}

void AutoFitSettings::on_button_lccValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::lccAF, dst->pars->getParameterValue(Enumerations::lcc, Enumerations::val));
    ui->lccValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::lccAF),'f',5));
    if ( ui->lcc->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMax ) ) )
        ui->lccValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->lccValue->setStyleSheet("");
}

void AutoFitSettings::on_lccMin_editingFinished(){
    QString *tempText = new QString(ui->lccMin->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::lccAF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::lccAF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::lccAF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::lccAF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::lccAF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->lccValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMin){
        ui->lccMin->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].max) )
            ui->lccValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->lccMin->setText(tr("kein"));
}

void AutoFitSettings::on_lccMax_editingFinished(){
    QString *tempText = new QString(ui->lccMax->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::lccAF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::lccAF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::lccAF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::lccAF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::lccAF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->lccValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].useMax){
        ui->lccMax->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::lccAF].min))
            ui->lccValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->lccMax->setText(tr("kein"));
}


void AutoFitSettings::on_q_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->qValue->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::qAF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->qMin->show();
        ui->label_qMin->show();
        ui->qMax->show();
        ui->label_qMax->show();
        ui->label_qValue->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::qAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::qAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMax ) )
            ui->qValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_qValue->setText(tr("Wert"));
        ui->qMin->hide();
        ui->label_qMin->hide();
        ui->qMax->hide();
        ui->label_qMax->hide();
    }
    must_saveSequence = true;
    ui->q->setChecked(checked);
}

void AutoFitSettings::on_button_qValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::qAF, dst->pars->getParameterValue(Enumerations::q, Enumerations::val));
    ui->qValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::qAF),'f',5));
    if ( ui->q->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::qAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::qAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMax ) ) )
        ui->qValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->qValue->setStyleSheet("");
}

void AutoFitSettings::on_qMin_editingFinished(){
    QString *tempText = new QString(ui->qMin->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::qAF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::qAF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::qAF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::qAF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::qAF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->qValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMin){
        ui->qMin->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::qAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::qAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].max) )
            ui->qValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->qMin->setText(tr("kein"));
}

void AutoFitSettings::on_qMax_editingFinished(){
    QString *tempText = new QString(ui->qMax->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::qAF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::qAF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::qAF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::qAF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::qAF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->qValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].useMax){
        ui->qMax->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::qAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::qAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::qAF].min))
            ui->qValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->qMax->setText(tr("kein"));
}


void AutoFitSettings::on_dan_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->danValue->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::danAF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->danMin->show();
        ui->label_danMin->show();
        ui->danMax->show();
        ui->label_danMax->show();
        ui->label_danValue->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::danAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::danAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMax ) )
            ui->danValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_danValue->setText(tr("Wert"));
        ui->danMin->hide();
        ui->label_danMin->hide();
        ui->danMax->hide();
        ui->label_danMax->hide();
    }
    must_saveSequence = true;
    ui->dan->setChecked(checked);
}

void AutoFitSettings::on_button_danValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::danAF, dst->pars->getParameterValue(Enumerations::dan, Enumerations::val));
    ui->danValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::danAF),'f',5));
    if ( ui->dan->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::danAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::danAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMax ) ) )
        ui->danValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->danValue->setStyleSheet("");
}

void AutoFitSettings::on_danMin_editingFinished(){
    QString *tempText = new QString(ui->danMin->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::danAF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::danAF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::danAF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::danAF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::danAF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->danValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMin){
        ui->danMin->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::danAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::danAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].max) )
            ui->danValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->danMin->setText(tr("kein"));
}

void AutoFitSettings::on_danMax_editingFinished(){
    QString *tempText = new QString(ui->danMax->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::danAF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::danAF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::danAF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::danAF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::danAF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->danValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].useMax){
        ui->danMax->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::danAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::danAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::danAF].min))
            ui->danValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->danMax->setText(tr("kein"));
}


void AutoFitSettings::on_k_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->kValue->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::kAF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->kMin->show();
        ui->label_kMin->show();
        ui->kMax->show();
        ui->label_kMax->show();
        ui->label_kValue->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::kAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::kAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMax ) )
            ui->kValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_kValue->setText(tr("Wert"));
        ui->kMin->hide();
        ui->label_kMin->hide();
        ui->kMax->hide();
        ui->label_kMax->hide();
    }
    must_saveSequence = true;
    ui->k->setChecked(checked);
}

void AutoFitSettings::on_button_kValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::kAF, dst->pars->getParameterValue(Enumerations::k, Enumerations::val));
    ui->kValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::kAF),'f',5));
    if ( ui->k->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::kAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::kAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMax ) ) )
        ui->kValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->kValue->setStyleSheet("");
}

void AutoFitSettings::on_kMin_editingFinished(){
    QString *tempText = new QString(ui->kMin->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::kAF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::kAF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::kAF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::kAF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::kAF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->kValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMin){
        ui->kMin->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::kAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::kAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].max) )
            ui->kValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->kMin->setText(tr("kein"));
}

void AutoFitSettings::on_kMax_editingFinished(){
    QString *tempText = new QString(ui->kMax->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::kAF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::kAF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::kAF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::kAF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::kAF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->kValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].useMax){
        ui->kMax->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::kAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::kAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::kAF].min))
            ui->kValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->kMax->setText(tr("kein"));
}


void AutoFitSettings::on_const1_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->const1Value->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const1AF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->const1Min->show();
        ui->label_const1Min->show();
        ui->const1Max->show();
        ui->label_const1Max->show();
        ui->label_const1Value->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMax ) )
            ui->const1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_const1Value->setText(tr("Wert"));
        ui->const1Min->hide();
        ui->label_const1Min->hide();
        ui->const1Max->hide();
        ui->label_const1Max->hide();
    }
    must_saveSequence = true;
    ui->const1->setChecked(checked);
}

void AutoFitSettings::on_button_const1ValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::const1AF, dst->pars->getParameterValue(Enumerations::const1, Enumerations::val));
    ui->const1Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::const1AF),'f',5));
    if ( ui->const1->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMax ) ) )
        ui->const1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->const1Value->setStyleSheet("");
}

void AutoFitSettings::on_const1Min_editingFinished(){
    QString *tempText = new QString(ui->const1Min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const1AF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const1AF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const1AF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const1AF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const1AF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->const1Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMin){
        ui->const1Min->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].max) )
            ui->const1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->const1Min->setText(tr("kein"));
}

void AutoFitSettings::on_const1Max_editingFinished(){
    QString *tempText = new QString(ui->const1Max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const1AF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const1AF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const1AF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const1AF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const1AF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->const1Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].useMax){
        ui->const1Max->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const1AF].min))
            ui->const1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->const1Max->setText(tr("kein"));
}


void AutoFitSettings::on_const2_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->const2Value->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const2AF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->const2Min->show();
        ui->label_const2Min->show();
        ui->const2Max->show();
        ui->label_const2Max->show();
        ui->label_const2Value->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMax ) )
            ui->const2Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_const2Value->setText(tr("Wert"));
        ui->const2Min->hide();
        ui->label_const2Min->hide();
        ui->const2Max->hide();
        ui->label_const2Max->hide();
    }
    must_saveSequence = true;
    ui->const2->setChecked(checked);
}

void AutoFitSettings::on_button_const2ValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::const2AF, dst->pars->getParameterValue(Enumerations::const2, Enumerations::val));
    ui->const2Value->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::const2AF),'f',5));
    if ( ui->const2->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMax ) ) )
        ui->const2Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->const2Value->setStyleSheet("");
}

void AutoFitSettings::on_const2Min_editingFinished(){
    QString *tempText = new QString(ui->const2Min->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const2AF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const2AF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const2AF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const2AF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const2AF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->const2Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMin){
        ui->const2Min->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].max) )
            ui->const2Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->const2Min->setText(tr("kein"));
}

void AutoFitSettings::on_const2Max_editingFinished(){
    QString *tempText = new QString(ui->const2Max->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const2AF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const2AF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const2AF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const2AF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::const2AF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->const2Value->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].useMax){
        ui->const2Max->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::const2AF].min))
            ui->const2Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->const2Max->setText(tr("kein"));
}


void AutoFitSettings::on_g_clicked(bool checked){
    if ( (!(checked)) && ( (activeParameters - 1) == 0) ){
        checked = true;
        --activeParameters;
    }
    ui->gValue->setStyleSheet("");
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].autoFit = checked;
    if (checked){
        ++activeParameters;
        ui->gMin->show();
        ui->label_gMin->show();
        ui->gMax->show();
        ui->label_gMax->show();
        ui->label_gValue->setText(tr("(Start)Wert"));
        if ( ( (dst->parsAutoFit->getParameterValue(Enumerations::gAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::gAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMax ) )
            ui->gValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else{
        --activeParameters;
        ui->label_gValue->setText(tr("Wert"));
        ui->gMin->hide();
        ui->label_gMin->hide();
        ui->gMax->hide();
        ui->label_gMax->hide();
    }
    must_saveSequence = true;
    ui->g->setChecked(checked);
}

void AutoFitSettings::on_button_gValueTakeover_clicked(){
    dst->parsAutoFit->setParameterValue(Enumerations::gAF, dst->pars->getParameterValue(Enumerations::g, Enumerations::val));
    ui->gValue->setText(QString::number(dst->parsAutoFit->getParameterValue(Enumerations::gAF),'f',5));
    if ( ui->g->isChecked() && ( ( (dst->parsAutoFit->getParameterValue(Enumerations::gAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].min) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMin ) || ( (dst->parsAutoFit->getParameterValue(Enumerations::gAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].max) &&  afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMax ) ) )
        ui->gValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    else
        ui->gValue->setStyleSheet("");
}

void AutoFitSettings::on_gMin_editingFinished(){
    QString *tempText = new QString(ui->gMin->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp <= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].max)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].min = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].min = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].max;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].useMin = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMin){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].useMin = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMin)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].useMin = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->gValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMin){
        ui->gMin->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].min,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::gAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].min) || (dst->parsAutoFit->getParameterValue(Enumerations::gAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].max) )
            ui->gValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->gMin->setText(tr("kein"));
}

void AutoFitSettings::on_gMax_editingFinished(){
    QString *tempText = new QString(ui->gMax->text());
    if (AuxiliaryMethods::isDouble(tempText)){
        double temp = AuxiliaryMethods::round(tempText->toDouble(), 5);
        if (temp >= afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].min)
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].max = temp;
        else
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].max = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].min;
        (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].useMax = true;
        must_saveSequence = true;
    }
    else
        if (( tempText->compare("n", Qt::CaseInsensitive) == 0) && afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMax){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].useMax = false;
            must_saveSequence = true;
        }
        else
            if (( tempText->compare("u", Qt::CaseInsensitive) == 0) && !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMax)){
                (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].useMax = true;
                must_saveSequence = true;
            }
    delete tempText;
    ui->gValue->setStyleSheet("");
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].useMax){
        ui->gMax->setText(QString::number(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].max,'f',5));
        if ( (dst->parsAutoFit->getParameterValue(Enumerations::gAF) > afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].max) || (dst->parsAutoFit->getParameterValue(Enumerations::gAF) < afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].min))
            ui->gValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->gMax->setText(tr("kein"));
}


void AutoFitSettings::on_buttonTakeOverResults_clicked(){
    dst->pars->setParameterValue(Enumerations::cno, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::cnoAF), false);
    dst->pars->setParameterValue(Enumerations::mu, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::muAF), false);
    dst->pars->setParameterValue(Enumerations::beta, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::betaAF), false);
    dst->pars->setParameterValue(Enumerations::a3, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::a3AF), false);
    dst->pars->setParameterValue(Enumerations::da3, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::da3AF), false);
    dst->pars->setParameterValue(Enumerations::sig3, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::sig3AF), false);
    dst->pars->setParameterValue(Enumerations::u3, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::u3AF), false);
    dst->pars->setParameterValue(Enumerations::eta, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::etaAF), false);
    dst->pars->setParameterValue(Enumerations::nu, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::nuAF), false);
    dst->pars->setParameterValue(Enumerations::alpha, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::alphaAF), false);
    dst->pars->setParameterValue(Enumerations::sig1, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::sig1AF), false);
    dst->pars->setParameterValue(Enumerations::lcc, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::lccAF), false);
    dst->pars->setParameterValue(Enumerations::q, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::qAF), false);
    dst->pars->setParameterValue(Enumerations::dan, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::danAF), false);
    dst->pars->setParameterValue(Enumerations::k, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::kAF), false);
    dst->pars->setParameterValue(Enumerations::const1, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::const1AF), false);
    dst->pars->setParameterValue(Enumerations::const2, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::const2AF), false);
    dst->pars->setParameterValue(Enumerations::g, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::gAF), false);
    must_takeOverResults = false;
    mw->takeOver_afsResults();

}

void AutoFitSettings::on_buttonDisplayResults_clicked(bool checked){
    if (checked){
        dst->plotPars_MainWindow = dst->pars->duplicate();
        dst->plotPars_MainWindow->setParameterValue(Enumerations::cno, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::cnoAF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::mu, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::muAF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::beta, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::betaAF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::a3, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::a3AF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::da3, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::da3AF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::sig3, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::sig3AF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::u3, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::u3AF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::eta, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::etaAF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::nu, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::nuAF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::alpha, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::alphaAF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::sig1, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::sig1AF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::lcc, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::lccAF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::q, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::qAF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::dan, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::danAF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::k, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::kAF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::const1, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::const1AF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::const2, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::const2AF), false);
        dst->plotPars_MainWindow->setParameterValue(Enumerations::g, Enumerations::val, dst->parsAutoFit->getParameterValue(Enumerations::gAF), false);
        dst->plotPars_Deviation = dst->plotPars_MainWindow;
    }
    else{
        delete dst->plotPars_Deviation;
        dst->plotPars_Deviation = dst->pars;
        dst->plotPars_MainWindow = dst->pars;
    }
    mw->display_afsResults(checked);
}

void AutoFitSettings::on_buttonTakeOverValues_clicked(){
    delete dst->parsAutoFit;
    dst->parsAutoFit = dst->pars->duplicateForAutoFit();
    setParameterWidgetValues();
    ui->buttonDisplayResults->setDisabled(true);
    ui->buttonTakeOverResults->setDisabled(true);
    must_takeOverResults = false;
}


void AutoFitSettings::on_buttonDoAutoFit_clicked(){
    switch (ui->comboBox_AutoFitProgram->currentIndex()){
        case 0:{ //if "wstp.h" is not available, some other class is to be used for automatic fitting
            if (QFile::exists(*(sst->prgset->kernelPath))){
                bool useGradient_Temp[afseq->numberOfSteps];
                if (!(dst->prgpar->useGradient))
                    for (int i = 0; i < afseq->numberOfSteps; ++i){
                        useGradient_Temp[i] = afseq->autoFitSteps->at(i).autoFitElements[Enumerations::gAF].autoFit;
                        (afseq->autoFitSteps->data())[i].autoFitElements[Enumerations::gAF].autoFit = false;
                    }
                //if "wstp.h" is not available, the next three lines must be adapted
                AutoFitMathematica *autoFit = new AutoFitMathematica(dst, sst, ui->spinBox_dataThinningFactor->value());
                autoFit->autoFit(dst->parsAutoFit, afseq, 0, dst->parsNamesAutoFit, dst->prgpar->par_b, dst->pars->getParameterValue(Enumerations::cN, Enumerations::val), dst->pars->getParameterValue(Enumerations::cO, Enumerations::val));
                delete autoFit;
                if (!(dst->prgpar->useGradient))
                    for (int i = 0; i < afseq->numberOfSteps; ++i){
                        (afseq->autoFitSteps->data())[i].autoFitElements[Enumerations::gAF].autoFit = useGradient_Temp[i];
                    }
                setParameterWidgetValues();
                ui->listWidget_AutoFitSteps->setCurrentRow(ui->listWidget_AutoFitSteps->count() - 1);
                ui->buttonDisplayResults->setEnabled(true);
                ui->buttonTakeOverResults->setEnabled(true);
                if (ui->buttonDisplayResults->isChecked()){
                    delete dst->plotPars_MainWindow;
                    on_buttonDisplayResults_clicked(true);
                }
                must_takeOverResults = true;
            }
            else
                AuxiliaryMethods::errorMessage(this, new QString(tr("Fehler: Mathematica-Kernel nicht gefunden!\nDie automatische Anpassung ist nicht moeglich.")));
            break;
        }
        default:
            break;
    }
}


//AutoFit sequences

void AutoFitSettings::on_comboBox_AutoFitSequences_currentIndexChanged(int index){ //ignore unsued variable warning
    delete afseq;
    afseq = new AutoFitSequence();
    if (!(pfa->readAFS(afseq, new QString (ui->comboBox_AutoFitSequences->currentData().toString()), this->dst))){
        setDefaultAutoFitSequence();
        bool prevState = ui->comboBox_AutoFitSequences->blockSignals(true); //save previous state
        ui->comboBox_AutoFitSequences->removeItem(index);
        ui->comboBox_AutoFitSequences->setCurrentIndex(-1);
        ui->comboBox_AutoFitSequences->blockSignals(prevState); //restore previous state
        ui->buttonSaveAutoFitSequence->setDisabled(true);
        ui->buttonRenameAutoFitSequence->setDisabled(true);
        ui->buttonDeleteAutoFitSequence->setDisabled(true);
    }
    else{
        ui->buttonSaveAutoFitSequence->setEnabled(true);
        ui->buttonRenameAutoFitSequence->setEnabled(true);
        ui->buttonDeleteAutoFitSequence->setEnabled(true);
        delete afseq->name;
        afseq->name = new QString(ui->comboBox_AutoFitSequences->currentText());
    }

    //set entries of listWidget AutoFitSteps and select first step
    activeSteps = 0;
    bool prevState = ui->listWidget_AutoFitSteps->blockSignals(true); //save previous state
    ui->listWidget_AutoFitSteps->clear();
    for (int i = 0; i < afseq->numberOfSteps; ++i){
        ui->listWidget_AutoFitSteps->addItem(*(afseq->autoFitSteps->at(i).name));
        ui->listWidget_AutoFitSteps->item(i)->setFlags((ui->listWidget_AutoFitSteps->item(i)->flags()) & ~Qt::ItemIsUserCheckable & ~Qt::ItemIsDragEnabled);
        if (afseq->autoFitSteps->at(i).useStep){
            ui->listWidget_AutoFitSteps->item(i)->setBackground(*brush_activeStep);
            ++activeSteps;
        }
        else
            ui->listWidget_AutoFitSteps->item(i)->setBackground(*brush_inactiveStep);
    }
    ui->listWidget_AutoFitSteps->blockSignals(prevState); //restore previous state
    ui->listWidget_AutoFitSteps->setCurrentRow(0);
    if (ui->listWidget_AutoFitSteps->count() == 1) //at least one step is always set
        ui->buttonDeleteAutoFitStep->setDisabled(true);
    else
        ui->buttonDeleteAutoFitStep->setEnabled(true);

    ui->buttonTakeOverValues->click();

    must_saveSequence = false;
}

void AutoFitSettings::on_buttonAddAutoFitSequence_clicked(){
    saveAutoFitSequenceMaybe();
    Dialog_name *dlg = new Dialog_name(new QString(tr("Name der Autofit-Sequenz eingeben")));
    dlg->exec();
    if ( !(dlg->getName()->isEmpty()) ){
        if (!(existsSavePath(dlg->getName()))){
            delete afseq->name;
            afseq->name = dlg->getName();
            QString *savePath = new QString(*(afseq->name));
            savePath->replace(" ", "_", Qt::CaseInsensitive);
            savePath->replace("/", "_", Qt::CaseInsensitive);
            savePath->replace("\\", "_", Qt::CaseInsensitive);
            savePath->replace(":", "_", Qt::CaseInsensitive);
            savePath->replace("*", "_", Qt::CaseInsensitive);
            savePath->replace("?", "_", Qt::CaseInsensitive);
            savePath->replace("\"", "_", Qt::CaseInsensitive);
            savePath->replace("<", "_", Qt::CaseInsensitive);
            savePath->replace(">", "_", Qt::CaseInsensitive);
            savePath->replace("|", "_", Qt::CaseInsensitive);
            savePath->replace(QRegExp("_{2,}"), "_");
            savePath->insert(0, *(dst->applicationDirPath) + "/data/autofitsequences/");
            savePath->append(".afs");
            QString *savePathCopy = new QString(*savePath); //copy needed as savePath is deleted in writeAFS
            if (pfa->writeAFS(afseq, savePath, dst)){
                ui->comboBox_AutoFitSequences->addItem(*(afseq->name), *(savePathCopy));
                ui->comboBox_AutoFitSequences->setCurrentIndex(ui->comboBox_AutoFitSequences->count() - 1);
            }
            else{
                AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern der Autofit-Sequenz fehlgeschlagen.")));
                delete afseq->name;
                afseq->name = new QString(ui->comboBox_AutoFitSequences->currentText());
            }
            delete savePathCopy;
        }
        else
            AuxiliaryMethods::errorMessage(this, new QString(tr("Eintrag und oder Datei bereits vorhanden.")));
    }
        delete dlg;
}

void AutoFitSettings::on_buttonSaveAutoFitSequence_clicked(){
    if (ui->comboBox_AutoFitSequences->currentIndex() >= 0){
        QString *savePath = new QString(ui->comboBox_AutoFitSequences->currentText());
        savePath->replace(" ", "_", Qt::CaseInsensitive);
        savePath->replace("/", "_", Qt::CaseInsensitive);
        savePath->replace("\\", "_", Qt::CaseInsensitive);
        savePath->replace(":", "_", Qt::CaseInsensitive);
        savePath->replace("*", "_", Qt::CaseInsensitive);
        savePath->replace("?", "_", Qt::CaseInsensitive);
        savePath->replace("\"", "_", Qt::CaseInsensitive);
        savePath->replace("<", "_", Qt::CaseInsensitive);
        savePath->replace(">", "_", Qt::CaseInsensitive);
        savePath->replace("|", "_", Qt::CaseInsensitive);
        savePath->replace(QRegExp("_{2,}"), "_");
        savePath->insert(0, *(dst->applicationDirPath) + "/data/autofitsequences/");
        savePath->append(".afs");
        if ( (savePath->compare(ui->comboBox_AutoFitSequences->currentData().toString(), Qt::CaseInsensitive)) != 0){
            QFile *file = new QFile(ui->comboBox_AutoFitSequences->currentData().toString());
            file->remove();
            delete file;
            ui->comboBox_AutoFitSequences->setItemData(ui->comboBox_AutoFitSequences->currentIndex(), *(savePath));
        }
        delete afseq->name;
        afseq->name = new QString(ui->comboBox_AutoFitSequences->currentText());
        if (!(pfa->writeAFS(afseq, savePath, dst)))
            AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern der Autofit-Sequenz fehlgeschlagen.")));
        else
            must_saveSequence = false;
    }
}

void AutoFitSettings::on_buttonRenameAutoFitSequence_clicked(){
    if (ui->comboBox_AutoFitSequences->currentIndex() >= 0){
        Dialog_name *dlg = new Dialog_name(new QString(tr("Name der Autofit-Sequenz eingeben")), new QString(ui->comboBox_AutoFitSequences->currentText()));
        dlg->exec();
        if ( ! ( (dlg->getName()->isEmpty()) ||  (dlg->getName()->compare(ui->comboBox_AutoFitSequences->currentText(), Qt::CaseInsensitive)) == 0) ){
            if (! (existsSavePath(dlg->getName()))){
                ui->comboBox_AutoFitSequences->setItemText(ui->comboBox_AutoFitSequences->currentIndex(), *(dlg->getName()));
                must_saveSequence = true;
            }
            else
                AuxiliaryMethods::errorMessage(this, new QString(tr("Eintrag und oder Datei bereits vorhanden.")));
        }
        delete dlg->getName();
        delete dlg;
    }
}

void AutoFitSettings::on_buttonDeleteAutoFitSequence_clicked(){
    if (ui->comboBox_AutoFitSequences->currentIndex() >= 0){
        QFile *file = new QFile(ui->comboBox_AutoFitSequences->currentData().toString());
        file->remove();
        delete file;
        ui->comboBox_AutoFitSequences->removeItem(ui->comboBox_AutoFitSequences->currentIndex());
    }
}


//AutoFit steps

void AutoFitSettings::on_listWidget_AutoFitSteps_currentRowChanged(int currentRow){

    activeParameters = 0;


    ui->cnoValue->setStyleSheet("");
    ui->cno->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::cnoAF].autoFit);
    if (ui->cno->isChecked()){
        ++activeParameters;
        ui->cnoMin->show();
        ui->label_cnoMin->show();
        ui->cnoMax->show();
        ui->label_cnoMax->show();
        ui->label_cnoValue->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_cnoValue->setText(tr("Wert"));
        ui->cnoMin->hide();
        ui->label_cnoMin->hide();
        ui->cnoMax->hide();
        ui->label_cnoMax->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::cnoAF].useMin){
        ui->cnoMin->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::cnoAF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::cnoAF].min && ui->cno->isChecked())
            ui->cnoValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->cnoMin->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::cnoAF].useMax){
        ui->cnoMax->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::cnoAF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::cnoAF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::cnoAF].max && ui->cno->isChecked())
            ui->cnoValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->cnoMax->setText(tr("kein"));

    ui->muValue->setStyleSheet("");
    ui->mu->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::muAF].autoFit);
    if (ui->mu->isChecked()){
        ++activeParameters;
        ui->muMin->show();
        ui->label_muMin->show();
        ui->muMax->show();
        ui->label_muMax->show();
        ui->label_muValue->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_muValue->setText(tr("Wert"));
        ui->muMin->hide();
        ui->label_muMin->hide();
        ui->muMax->hide();
        ui->label_muMax->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::muAF].useMin){
        ui->muMin->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::muAF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::muAF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::muAF].min && ui->mu->isChecked())
            ui->muValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->muMin->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::muAF].useMax){
        ui->muMax->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::muAF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::muAF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::muAF].max && ui->mu->isChecked())
            ui->muValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->muMax->setText(tr("kein"));

    ui->betaValue->setStyleSheet("");
    ui->beta->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::betaAF].autoFit);
    if (ui->beta->isChecked()){
        ++activeParameters;
        ui->betaMin->show();
        ui->label_betaMin->show();
        ui->betaMax->show();
        ui->label_betaMax->show();
        ui->label_betaValue->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_betaValue->setText(tr("Wert"));
        ui->betaMin->hide();
        ui->label_betaMin->hide();
        ui->betaMax->hide();
        ui->label_betaMax->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::betaAF].useMin){
        ui->betaMin->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::betaAF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::betaAF].min && ui->beta->isChecked())
            ui->betaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->betaMin->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::betaAF].useMax){
        ui->betaMax->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::betaAF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::betaAF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::betaAF].max && ui->beta->isChecked())
            ui->betaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->betaMax->setText(tr("kein"));

    ui->a3Value->setStyleSheet("");
    ui->a3->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::a3AF].autoFit);
    if (ui->a3->isChecked()){
        ++activeParameters;
        ui->a3Min->show();
        ui->label_a3Min->show();
        ui->a3Max->show();
        ui->label_a3Max->show();
        ui->label_a3Value->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_a3Value->setText(tr("Wert"));
        ui->a3Min->hide();
        ui->label_a3Min->hide();
        ui->a3Max->hide();
        ui->label_a3Max->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::a3AF].useMin){
        ui->a3Min->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::a3AF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::a3AF].min && ui->a3->isChecked())
            ui->a3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->a3Min->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::a3AF].useMax){
        ui->a3Max->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::a3AF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::a3AF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::a3AF].max && ui->a3->isChecked())
            ui->a3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->a3Max->setText(tr("kein"));

    ui->da3Value->setStyleSheet("");
    ui->da3->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::da3AF].autoFit);
    if (ui->da3->isChecked()){
        ++activeParameters;
        ui->da3Min->show();
        ui->label_da3Min->show();
        ui->da3Max->show();
        ui->label_da3Max->show();
        ui->label_da3Value->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_da3Value->setText(tr("Wert"));
        ui->da3Min->hide();
        ui->label_da3Min->hide();
        ui->da3Max->hide();
        ui->label_da3Max->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::da3AF].useMin){
        ui->da3Min->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::da3AF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::da3AF].min && ui->da3->isChecked())
            ui->da3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->da3Min->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::da3AF].useMax){
        ui->da3Max->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::da3AF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::da3AF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::da3AF].max && ui->da3->isChecked())
            ui->da3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->da3Max->setText(tr("kein"));

    ui->sig3Value->setStyleSheet("");
    ui->sig3->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig3AF].autoFit);
    if (ui->sig3->isChecked()){
        ++activeParameters;
        ui->sig3Min->show();
        ui->label_sig3Min->show();
        ui->sig3Max->show();
        ui->label_sig3Max->show();
        ui->label_sig3Value->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_sig3Value->setText(tr("Wert"));
        ui->sig3Min->hide();
        ui->label_sig3Min->hide();
        ui->sig3Max->hide();
        ui->label_sig3Max->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig3AF].useMin){
        ui->sig3Min->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig3AF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig3AF].min && ui->sig3->isChecked())
            ui->sig3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->sig3Min->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig3AF].useMax){
        ui->sig3Max->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig3AF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::sig3AF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig3AF].max && ui->sig3->isChecked())
            ui->sig3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->sig3Max->setText(tr("kein"));

    ui->u3Value->setStyleSheet("");
    ui->u3->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::u3AF].autoFit);
    if (ui->u3->isChecked()){
        ++activeParameters;
        ui->u3Min->show();
        ui->label_u3Min->show();
        ui->u3Max->show();
        ui->label_u3Max->show();
        ui->label_u3Value->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_u3Value->setText(tr("Wert"));
        ui->u3Min->hide();
        ui->label_u3Min->hide();
        ui->u3Max->hide();
        ui->label_u3Max->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::u3AF].useMin){
        ui->u3Min->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::u3AF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::u3AF].min && ui->u3->isChecked())
            ui->u3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->u3Min->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::u3AF].useMax){
        ui->u3Max->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::u3AF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::u3AF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::u3AF].max && ui->u3->isChecked())
            ui->u3Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->u3Max->setText(tr("kein"));

    ui->etaValue->setStyleSheet("");
    ui->eta->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::etaAF].autoFit);
    if (ui->eta->isChecked()){
        ++activeParameters;
        ui->etaMin->show();
        ui->label_etaMin->show();
        ui->etaMax->show();
        ui->label_etaMax->show();
        ui->label_etaValue->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_etaValue->setText(tr("Wert"));
        ui->etaMin->hide();
        ui->label_etaMin->hide();
        ui->etaMax->hide();
        ui->label_etaMax->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::etaAF].useMin){
        ui->etaMin->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::etaAF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::etaAF].min && ui->eta->isChecked())
            ui->etaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->etaMin->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::etaAF].useMax){
        ui->etaMax->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::etaAF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::etaAF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::etaAF].max && ui->eta->isChecked())
            ui->etaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->etaMax->setText(tr("kein"));

    ui->nuValue->setStyleSheet("");
    ui->nu->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::nuAF].autoFit);
    if (ui->nu->isChecked()){
        ++activeParameters;
        ui->nuMin->show();
        ui->label_nuMin->show();
        ui->nuMax->show();
        ui->label_nuMax->show();
        ui->label_nuValue->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_nuValue->setText(tr("Wert"));
        ui->nuMin->hide();
        ui->label_nuMin->hide();
        ui->nuMax->hide();
        ui->label_nuMax->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::nuAF].useMin){
        ui->nuMin->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::nuAF].min,'f',0));
        if (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::nuAF].min && ui->nu->isChecked())
            ui->nuValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->nuMin->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::nuAF].useMax){
        ui->nuMax->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::nuAF].max,'f',0));
        if (dst->parsAutoFit->getParameterValue(Enumerations::nuAF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::nuAF].max && ui->nu->isChecked())
            ui->nuValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->nuMax->setText(tr("kein"));

    ui->alphaValue->setStyleSheet("");
    ui->alpha->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::alphaAF].autoFit);
    if (ui->alpha->isChecked()){
        ++activeParameters;
        ui->alphaMin->show();
        ui->label_alphaMin->show();
        ui->alphaMax->show();
        ui->label_alphaMax->show();
        ui->label_alphaValue->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_alphaValue->setText(tr("Wert"));
        ui->alphaMin->hide();
        ui->label_alphaMin->hide();
        ui->alphaMax->hide();
        ui->label_alphaMax->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::alphaAF].useMin){
        ui->alphaMin->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::alphaAF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::alphaAF].min && ui->alpha->isChecked())
            ui->alphaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->alphaMin->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::alphaAF].useMax){
        ui->alphaMax->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::alphaAF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::alphaAF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::alphaAF].max && ui->alpha->isChecked())
            ui->alphaValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->alphaMax->setText(tr("kein"));

    ui->sig1Value->setStyleSheet("");
    ui->sig1->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig1AF].autoFit);
    if (ui->sig1->isChecked()){
        ++activeParameters;
        ui->sig1Min->show();
        ui->label_sig1Min->show();
        ui->sig1Max->show();
        ui->label_sig1Max->show();
        ui->label_sig1Value->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_sig1Value->setText(tr("Wert"));
        ui->sig1Min->hide();
        ui->label_sig1Min->hide();
        ui->sig1Max->hide();
        ui->label_sig1Max->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig1AF].useMin){
        ui->sig1Min->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig1AF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig1AF].min && ui->sig1->isChecked())
            ui->sig1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->sig1Min->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig1AF].useMax){
        ui->sig1Max->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig1AF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::sig1AF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::sig1AF].max && ui->sig1->isChecked())
            ui->sig1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->sig1Max->setText(tr("kein"));

    ui->lccValue->setStyleSheet("");
    ui->lcc->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::lccAF].autoFit);
    if (ui->lcc->isChecked()){
        ++activeParameters;
        ui->lccMin->show();
        ui->label_lccMin->show();
        ui->lccMax->show();
        ui->label_lccMax->show();
        ui->label_lccValue->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_lccValue->setText(tr("Wert"));
        ui->lccMin->hide();
        ui->label_lccMin->hide();
        ui->lccMax->hide();
        ui->label_lccMax->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::lccAF].useMin){
        ui->lccMin->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::lccAF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::lccAF].min && ui->lcc->isChecked())
            ui->lccValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->lccMin->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::lccAF].useMax){
        ui->lccMax->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::lccAF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::lccAF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::lccAF].max && ui->lcc->isChecked())
            ui->lccValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->lccMax->setText(tr("kein"));

    ui->qValue->setStyleSheet("");
    ui->q->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::qAF].autoFit);
    if (ui->q->isChecked()){
        ++activeParameters;
        ui->qMin->show();
        ui->label_qMin->show();
        ui->qMax->show();
        ui->label_qMax->show();
        ui->label_qValue->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_qValue->setText(tr("Wert"));
        ui->qMin->hide();
        ui->label_qMin->hide();
        ui->qMax->hide();
        ui->label_qMax->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::qAF].useMin){
        ui->qMin->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::qAF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::qAF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::qAF].min && ui->q->isChecked())
            ui->qValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->qMin->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::qAF].useMax){
        ui->qMax->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::qAF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::qAF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::qAF].max && ui->q->isChecked())
            ui->qValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->qMax->setText(tr("kein"));

    ui->danValue->setStyleSheet("");
    ui->dan->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::danAF].autoFit);
    if (ui->dan->isChecked()){
        ++activeParameters;
        ui->danMin->show();
        ui->label_danMin->show();
        ui->danMax->show();
        ui->label_danMax->show();
        ui->label_danValue->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_danValue->setText(tr("Wert"));
        ui->danMin->hide();
        ui->label_danMin->hide();
        ui->danMax->hide();
        ui->label_danMax->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::danAF].useMin){
        ui->danMin->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::danAF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::danAF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::danAF].min && ui->dan->isChecked())
            ui->danValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->danMin->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::danAF].useMax){
        ui->danMax->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::danAF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::danAF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::danAF].max && ui->dan->isChecked())
            ui->danValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->danMax->setText(tr("kein"));

    ui->kValue->setStyleSheet("");
    ui->k->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::kAF].autoFit);
    if (ui->k->isChecked()){
        ++activeParameters;
        ui->kMin->show();
        ui->label_kMin->show();
        ui->kMax->show();
        ui->label_kMax->show();
        ui->label_kValue->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_kValue->setText(tr("Wert"));
        ui->kMin->hide();
        ui->label_kMin->hide();
        ui->kMax->hide();
        ui->label_kMax->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::kAF].useMin){
        ui->kMin->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::kAF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::kAF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::kAF].min && ui->k->isChecked())
            ui->kValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->kMin->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::kAF].useMax){
        ui->kMax->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::kAF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::kAF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::kAF].max && ui->k->isChecked())
            ui->kValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->kMax->setText(tr("kein"));

    ui->const1Value->setStyleSheet("");
    ui->const1->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const1AF].autoFit);
    if (ui->const1->isChecked()){
        ++activeParameters;
        ui->const1Min->show();
        ui->label_const1Min->show();
        ui->const1Max->show();
        ui->label_const1Max->show();
        ui->label_const1Value->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_const1Value->setText(tr("Wert"));
        ui->const1Min->hide();
        ui->label_const1Min->hide();
        ui->const1Max->hide();
        ui->label_const1Max->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const1AF].useMin){
        ui->const1Min->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const1AF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const1AF].min && ui->const1->isChecked())
            ui->const1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->const1Min->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const1AF].useMax){
        ui->const1Max->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const1AF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::const1AF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const1AF].max && ui->const1->isChecked())
            ui->const1Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->const1Max->setText(tr("kein"));

    ui->const2Value->setStyleSheet("");
    ui->const2->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const2AF].autoFit);
    if (ui->const2->isChecked()){
        ++activeParameters;
        ui->const2Min->show();
        ui->label_const2Min->show();
        ui->const2Max->show();
        ui->label_const2Max->show();
        ui->label_const2Value->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_const2Value->setText(tr("Wert"));
        ui->const2Min->hide();
        ui->label_const2Min->hide();
        ui->const2Max->hide();
        ui->label_const2Max->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const2AF].useMin){
        ui->const2Min->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const2AF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const2AF].min && ui->const2->isChecked())
            ui->const2Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->const2Min->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const2AF].useMax){
        ui->const2Max->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const2AF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::const2AF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::const2AF].max && ui->const2->isChecked())
            ui->const2Value->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->const2Max->setText(tr("kein"));

    ui->gValue->setStyleSheet("");
    ui->g->setChecked(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::gAF].autoFit);
    if (ui->g->isChecked()){
        if (dst->prgpar->useGradient)
            ++activeParameters;
        ui->gMin->show();
        ui->label_gMin->show();
        ui->gMax->show();
        ui->label_gMax->show();
        ui->label_gValue->setText(tr("(Start)Wert"));
    }
    else{
        ui->label_gValue->setText(tr("Wert"));
        ui->gMin->hide();
        ui->label_gMin->hide();
        ui->gMax->hide();
        ui->label_gMax->hide();
    }
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::gAF].useMin){
        ui->gMin->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::gAF].min,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::gAF) < afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::gAF].min && ui->g->isChecked())
            ui->gValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->gMin->setText(tr("kein"));
    if (afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::gAF].useMax){
        ui->gMax->setText(QString::number(afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::gAF].max,'f',5));
        if (dst->parsAutoFit->getParameterValue(Enumerations::gAF) > afseq->autoFitSteps->at(currentRow).autoFitElements[Enumerations::gAF].max && ui->g->isChecked())
            ui->gValue->setStyleSheet("background-color: rgb(255, 255, 0);");
    }
    else
        ui->gMax->setText(tr("kein"));

    if (afseq->autoFitSteps->at(currentRow).useStep){
        ui->buttonDisableAutoFitStep->setText(tr("deaktivieren"));
        if (activeSteps == 1)
            ui->buttonDisableAutoFitStep->setDisabled(true);
    }
    else{
        ui->buttonDisableAutoFitStep->setText(tr("aktivieren"));
        if (activeSteps == 1)
            ui->buttonDisableAutoFitStep->setEnabled(true);
    }

    ui->listWidget_additionalConditions->clear();
    for (int i = 0; i < afseq->autoFitSteps->at(currentRow).additionalConditions->size(); ++i){
        ui->listWidget_additionalConditions->addItem(afseq->autoFitSteps->at(currentRow).additionalConditions->at(i));
        ui->listWidget_additionalConditions->item(i)->setFlags((ui->listWidget_additionalConditions->item(i)->flags()) & ~Qt::ItemIsUserCheckable & ~Qt::ItemIsDragEnabled);
    }
    if (afseq->autoFitSteps->at(currentRow).additionalConditions->size() > 0){ //only set first row if number of addtional conditions > 0
        ui->listWidget_additionalConditions->setCurrentRow(0);
        ui->buttonChangeAdditonalCondition->setEnabled(true);
        ui->buttonDeleteAdditonalCondition->setEnabled(true);
    }
    else{
        ui->buttonChangeAdditonalCondition->setDisabled(true);
        ui->buttonDeleteAdditonalCondition->setDisabled(true);
    }

    if (currentRow == 0)
        ui->buttonMoveAutoFitStepUp->setDisabled(true);
    else
        ui->buttonMoveAutoFitStepUp->setEnabled(true);

    if ((currentRow + 1) == ui->listWidget_AutoFitSteps->count())
        ui->buttonMoveAutoFitStepDown->setDisabled(true);
    else
        ui->buttonMoveAutoFitStepDown->setEnabled(true);
}

void AutoFitSettings::on_buttonMoveAutoFitStepUp_clicked(){
    short stepToMove = ui->listWidget_AutoFitSteps->currentRow(); //index of step to move = target index + 1
    AutoFitStep tempStep = afseq->autoFitSteps->takeAt(stepToMove);
    afseq->autoFitSteps->insert( (stepToMove - 1), tempStep);
    QListWidgetItem *tempItem = ui->listWidget_AutoFitSteps->takeItem(stepToMove);
    ui->listWidget_AutoFitSteps->insertItem( (stepToMove - 1), tempItem);
    ui->listWidget_AutoFitSteps->setCurrentRow(stepToMove - 1);
    must_saveSequence = true;
}

void AutoFitSettings::on_buttonMoveAutoFitStepDown_clicked(){
    short stepToMove = ui->listWidget_AutoFitSteps->currentRow(); //index of step to move = target index - 1
    AutoFitStep tempStep = afseq->autoFitSteps->takeAt(stepToMove);
    afseq->autoFitSteps->insert( (stepToMove + 1), tempStep);
    QListWidgetItem *tempItem = ui->listWidget_AutoFitSteps->takeItem(stepToMove);
    ui->listWidget_AutoFitSteps->insertItem( (stepToMove + 1), tempItem);
    ui->listWidget_AutoFitSteps->setCurrentRow(stepToMove + 1);
    must_saveSequence = true;
}

void AutoFitSettings::on_buttonAddAutoFitStep_clicked(){
    AutoFitStep *newStep = (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].duplicate();
    Dialog_name *dlg = new Dialog_name(new QString(tr("Name des Schritts eingeben")), new QString(ui->listWidget_AutoFitSteps->currentItem()->text()));
    dlg->exec();
    delete newStep->name;
    newStep->name = dlg->getName();
    delete dlg;
    newStep->useStep = true;
    afseq->autoFitSteps->append(*newStep);
    ++(afseq->numberOfSteps);
    delete newStep;
    ui->listWidget_AutoFitSteps->addItem(*(afseq->autoFitSteps->at(afseq->autoFitSteps->size() - 1).name));
    ui->listWidget_AutoFitSteps->item(afseq->autoFitSteps->size() - 1)->setFlags((ui->listWidget_AutoFitSteps->item(afseq->autoFitSteps->size() - 1)->flags()) & ~Qt::ItemIsUserCheckable & ~Qt::ItemIsDragEnabled);
    ui->listWidget_AutoFitSteps->item(afseq->autoFitSteps->size() - 1)->setBackground(*brush_activeStep);
    ++activeSteps;
    ui->buttonDeleteAutoFitStep->setEnabled(true);
    ui->buttonDisableAutoFitStep->setEnabled(true);
    ui->listWidget_AutoFitSteps->setCurrentRow(ui->listWidget_AutoFitSteps->count() - 1);
    must_saveSequence = true;
}

void AutoFitSettings::on_buttonRenameAutoFitStep_clicked(){
    if (ui->listWidget_AutoFitSteps->currentRow() >= 0){
        Dialog_name *dlg = new Dialog_name(new QString(tr("Name des Schritts eingeben")), new QString(ui->listWidget_AutoFitSteps->currentItem()->text()));
        dlg->exec();
        if ( ! ( (dlg->getName()->isEmpty()) ||  (dlg->getName()->compare(ui->listWidget_AutoFitSteps->currentItem()->text(), Qt::CaseInsensitive)) == 0) ){
            delete (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].name;
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].name = dlg->getName();
            ui->listWidget_AutoFitSteps->currentItem()->setText(*(dlg->getName()));
            must_saveSequence = true;
        }
        delete dlg;
    }
    must_saveSequence = true;
}

void AutoFitSettings::on_buttonDeleteAutoFitStep_clicked(){
    if (ui->listWidget_AutoFitSteps->count() > 1){
        short rowToDelete = ui->listWidget_AutoFitSteps->currentRow();
        bool prevState = ui->listWidget_AutoFitSteps->blockSignals(true); //save previous state
        delete ui->listWidget_AutoFitSteps->currentItem();
        ui->listWidget_AutoFitSteps->blockSignals(prevState); //restore previous state
        if (afseq->autoFitSteps->at(rowToDelete).useStep)
            --activeSteps;
        afseq->autoFitSteps->remove(rowToDelete);
        --(afseq->numberOfSteps);
        if (activeSteps == 0){
            (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].useStep = true;
            ui->listWidget_AutoFitSteps->currentItem()->setBackground(*brush_activeStep);
            ++activeSteps;
        }
        on_listWidget_AutoFitSteps_currentRowChanged(ui->listWidget_AutoFitSteps->currentRow());
        must_saveSequence = true;
    }
    if (ui->listWidget_AutoFitSteps->count() == 1){
        ui->buttonDeleteAutoFitStep->setDisabled(true);
        ui->buttonAddAutoFitStep->setFocus();
    }
}

void AutoFitSettings::on_buttonDisableAutoFitStep_clicked(){
    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].useStep = !(afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).useStep);
    if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).useStep){
        ui->buttonDisableAutoFitStep->setText(tr("deaktivieren"));
        ui->listWidget_AutoFitSteps->item(ui->listWidget_AutoFitSteps->currentRow())->setBackground(*(brush_activeStep));
        ++activeSteps;
    }
    else{
        ui->buttonDisableAutoFitStep->setText(tr("aktivieren"));
        ui->listWidget_AutoFitSteps->item(ui->listWidget_AutoFitSteps->currentRow())->setBackground(*(brush_inactiveStep));
        --activeSteps;
    }
    must_saveSequence = true;
}

void AutoFitSettings::on_buttonExecuteAutoFitStep_clicked(){
    switch (ui->comboBox_AutoFitProgram->currentIndex()){
        case 0:{ //if "wstp.h" is not available, some other class is to be used for automatic fitting
            if (QFile::exists(*(sst->prgset->kernelPath))){
                bool useGradient_Temp = true; //to avoid may be used uninitialized warning set to true
                if (!(dst->prgpar->useGradient)){
                    useGradient_Temp = afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).autoFitElements[Enumerations::gAF].autoFit;
                    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].autoFit = false;
                }
                //if "wstp.h" is not available, the next three lines must be adapted
                AutoFitMathematica *autoFit = new AutoFitMathematica(dst, sst, ui->spinBox_dataThinningFactor->value());
                autoFit->autoFit(dst->parsAutoFit, afseq, (ui->listWidget_AutoFitSteps->currentRow() + 1), dst->parsNamesAutoFit, dst->prgpar->par_b, dst->pars->getParameterValue(Enumerations::cN, Enumerations::val), dst->pars->getParameterValue(Enumerations::cO, Enumerations::val));
                delete autoFit;
                if (!(dst->prgpar->useGradient))
                    (afseq->autoFitSteps->data())[ui->listWidget_AutoFitSteps->currentRow()].autoFitElements[Enumerations::gAF].autoFit = useGradient_Temp;
                setParameterWidgetValues();
                ui->buttonDisplayResults->setEnabled(true);
                ui->buttonTakeOverResults->setEnabled(true);
                if (ui->buttonDisplayResults->isChecked()){
                    delete dst->plotPars_MainWindow;
                    on_buttonDisplayResults_clicked(true);
                }
                must_takeOverResults = true;
            }
            else
                AuxiliaryMethods::errorMessage(this, new QString(tr("Fehler: Mathematica-Kernel nicht gefunden!\nDie automatische Anpassung ist nicht moeglich.")));
            break;
        }
        default:
            break;
    }
}


//additional Condtions

void AutoFitSettings::on_buttonAddAdditonalCondition_clicked(){
    Dialog_name *dlg = new Dialog_name(new QString(tr("Bedingung eingeben")));
    dlg->exec();
    if (!(dlg->getName()->isEmpty())){
        afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).additionalConditions->append(*(dlg->getName()));
        ui->listWidget_additionalConditions->addItem(*(dlg->getName()));
        ui->listWidget_AutoFitSteps->item(ui->listWidget_additionalConditions->count() - 1)->setFlags((ui->listWidget_AutoFitSteps->item(ui->listWidget_additionalConditions->count() - 1)->flags()) & ~Qt::ItemIsUserCheckable & ~Qt::ItemIsDragEnabled);
        ui->listWidget_additionalConditions->setCurrentRow(ui->listWidget_additionalConditions->count() - 1); //select inserted item
        if (afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).additionalConditions->size() == 1){ //first addtional condition added
            ui->buttonChangeAdditonalCondition->setEnabled(true);
            ui->buttonDeleteAdditonalCondition->setEnabled(true);
        }
        must_saveSequence = true;
    }
    delete dlg->getName();
    delete dlg;
}

void AutoFitSettings::on_buttonChangeAdditonalCondition_clicked(){
    if (ui->listWidget_additionalConditions->currentRow() >= 0){
        Dialog_name *dlg = new Dialog_name(new QString(tr("Bedingung eingeben")), new QString(ui->listWidget_additionalConditions->currentItem()->text()));
        dlg->exec();
        if ( ! ( (dlg->getName()->isEmpty()) ||  (dlg->getName()->compare(ui->listWidget_additionalConditions->currentItem()->text(), Qt::CaseInsensitive)) == 0) ){
            afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).additionalConditions->replace(ui->listWidget_additionalConditions->currentRow(), *(dlg->getName()));
            ui->listWidget_additionalConditions->currentItem()->setText(*(dlg->getName()));
            must_saveSequence = true;
        }
        delete dlg->getName();
        delete dlg;
    }
}

void AutoFitSettings::on_buttonDeleteAdditonalCondition_clicked(){
    if (ui->listWidget_additionalConditions->currentRow() >= 0){
        afseq->autoFitSteps->at(ui->listWidget_AutoFitSteps->currentRow()).additionalConditions->removeAt(ui->listWidget_additionalConditions->currentRow());
        delete ui->listWidget_additionalConditions->currentItem();
        must_saveSequence = true;
    }
    if (ui->listWidget_additionalConditions->count() == 0){ //all addtional conditions deleted
        ui->buttonChangeAdditonalCondition->setDisabled(true);
        ui->buttonDeleteAdditonalCondition->setDisabled(true);
        ui->buttonAddAdditonalCondition->setFocus();
    }
}
